/****************************************************************************
*
*	Module Name: dcmapi.h
*
*	Description:
*
*	Revision History:
*	4.0	11/23/94		FJL
*			Made the following changes:
*			1.	Added conditional compilation of PRLOBYTE, PRHIBYTE, PRLOWORD,
*				and PRHIWORD definitions. Identifier "PRWIN" is used for Window's
*				compilation.
*
****************************************************************************/

/*
 * $Workfile:   dcmapi.h  $
 *
 * Current $Revision:   1.2  $ By $Author:   tharakan  $
 *
 * $Modtime:   18 May 1993 14:31:48  $
 *
 * Reverse Chronological Revision 
 *
 * $Log:   H:\penright\vcs\incs\dcmapi.h_v  $
 * 
 *    Rev 1.2   21 May 1993 16:36:10   tharakan
 * 
 * removed far from struct resp_entry.
 * moved DclChgRTS_DTRForOpen to DcmChgRTS_DTRForOpen
 * moved DclInpQSize to DcmInpDSize
 * 
 *    Rev 1.1   23 Apr 1993 17:00:30   tharakan
 * 
 * Changed __IN_PEN_API_H to __IN_PENRIGHT_H
 * 
 *    Rev 1.0   19 Apr 1993 15:56:04   tharakan
 * Initial Revision
 *
*/


/*
*       DCMAPI.H
*
*       PenRight! Datacomm (Dcm) API header file. (included by penapi.h)
*
*/

#ifndef __DCMAPI                        /* Prevent duplicate definition */
#define __DCMAPI

#include        <time.h>
#include        <stdio.h>


/******* Common definitions and typedefs ***********************************/

#define VOID                void

#define FAR                 _far
#define NEAR                _near
#define PASCAL              _pascal
#define CDECL               _cdecl


/****** Simple types & helper macros ***************************************/

typedef int                 BOOL;
#define FALSE               0
#define TRUE                1

typedef int                 INT;
typedef unsigned char       BYTE;
typedef unsigned short      WORD;
typedef unsigned long       DWORD;

typedef unsigned int        UINT;

#ifdef  STRICT
typedef signed long         LONG;
#else
#define LONG long
#endif  /* STRICT */

#ifdef PRWIN
#define PRLOBYTE(w)           ((BYTE)(w))
#define PRHIBYTE(w)           ((BYTE)(((UINT)(w) >> 8) & 0xFF))
#define PRLOWORD(l)           ((WORD)(DWORD)(l))
#define PRHIWORD(l)           ((WORD)((((DWORD)(l)) >> 16) & 0xFFFF))
#else
#define LOBYTE(w)           ((BYTE)(w))
#define HIBYTE(w)           ((BYTE)(((UINT)(w) >> 8) & 0xFF))
#define LOWORD(l)           ((WORD)(DWORD)(l))
#define HIWORD(l)           ((WORD)((((DWORD)(l)) >> 16) & 0xFFFF))
#endif

#define MAKELONG(low, high) ((LONG)(((WORD)(low)) | (((DWORD)((WORD)(high))) << 16)))

#ifndef NOMINMAX
#ifndef max
#define max(a,b)            (((a) > (b)) ? (a) : (b))
#endif
#ifndef min
#define min(a,b)            (((a) < (b)) ? (a) : (b))
#endif
#endif  /* NOMINMAX */


/****** Common pointer types ************************************************/

#ifndef NULL
#define NULL                0
#endif

typedef char NEAR*          PSTR;
typedef char NEAR*          NPSTR;


typedef char FAR*           LPSTR;
typedef const char FAR*     LPCSTR;

typedef BYTE NEAR*          PBYTE;
typedef BYTE FAR*           LPBYTE;

typedef int NEAR*           PINT;
typedef int FAR*            LPINT;

typedef WORD NEAR*          PWORD;
typedef WORD FAR*           LPWORD;

typedef long NEAR*          PLONG;
typedef long FAR*           LPLONG;

typedef DWORD NEAR*         PDWORD;
typedef DWORD FAR*          LPDWORD;

typedef void FAR*           LPVOID;


/****** PenRight! Dcm API Functions are always ...**************************/

#define PRDCAPI _cdecl


/****** Prototype for Cancel Function Parameter to Dcm APIs ****************/

typedef INT     (*P_CAN_FUNC)   (VOID); /* Pointer to function(void) which */
                                        /* returns an int */


/****** Error code returns from Dcm APIs ***********************************/

#define eDC_A_OK                0       /* Operation successful */
#define eDC_INV_PORT            10002   /* Port number out of range */
#define eDC_PORT_NOT_OPEN       10003   /* Port is not open */
#define eDC_INV_PARAMETER       10004   /* Invalid Level Zero parameter */

#define eDC_NO_PORT_FOUND       10006   /* No Port at specified addr */
#define eDC_OUT_Q_FULL          10007   /* Output queue filled */

#define eDC_PORT_OPEN_ALREADY   10009   /* Port already opened */
#define eDC_IN_Q_EMPTY          10010   /* Input queue is empty */

#define eDC_TIMEOUT             10012   /* Timeout waiting for event */
#define eDC_CANCEL              10013   /* Event cancel(ed) */
#define eDC_NO_PORT_ADS         10014   /* No port address available */
#define eDC_NO_IRQ_AVAIL        10015   /* No IRQ line available */
#define eDC_BAD_BUFFER          10016   /* Bad buffer size or buffer addr */
#define eDC_UNKNOWN_BAUD        10017   /* Unknown baud rate divisor */
#define eDC_INV_OPT_PARAM       10018   /* Invalid options parameter */

#define eDC_INTERNAL_ERROR      10099    

#define eDC_MEM_ALLOC_ERR       10050   /* Error allocating memory */
#define eDC_INV_IN_Q_SIZE       10051   /* Invalid input queue size */
#define eDC_INV_OUT_Q_SIZE      10052   /* Invalid output queue size */
#define eDC_INV_INT_VALUE       10053   /* Invalid interrupt level */
#define eDC_INV_BAUD_RATE       10054   /* Invalid baud rate */
#define eDC_INV_PARITY          10055   /* Invalid parity specified */
#define eDC_INV_DATA_BITS       10056   /* Invalid data bits specified */
#define eDC_INV_STOP_BITS       10057   /* Invalid stop bits specified */
#define eDC_INV_REMOTE_FLOW     10058   /* Invalid flow control option */
#define eDC_INV_LOCAL_FLOW      10059   /* Invalid flow control option */
#define eDC_INV_TRIMMING        10060   /* Invalid bit 7 trimming option */
#define eDC_INV_FORCING         10061   /* Invalid bit 7 forcing option */
#define eDC_INV_REQ_CTS         10062   /* Invalid CTS requirement opt. */
#define eDC_INV_BREAK_TIME      10063   /* Invalid break time specified */

#define eDC_HM_NOT_INIT         10100   /* Modem control return codes  */
#define eDC_HM_TRANS_ERR        10101
#define eDC_HM_NO_RESPONSE      10102
#define eDC_HM_CANNOT_RESET     10103
#define eDC_HM_UNEXPECTED_RESP  10104
#define eDC_HM_NO_CARRIER       10190   /* No DCD (also in filexfer) */

#define eDC_OPEN_FILE_ERR       10150   /* Error opening file */
#define eDC_EOF_ENCOUNTERED     10151   /* EOF while reading file */
#define eDC_READ_FILE_ERR       10152   /* Error reading send file */
#define eDC_WRITE_FILE_ERR      10153   /* Error writing receive file */
#define eDC_EXIST_FILE_ERR      10154   /* File overwrite abort */
#define eDC_UNLINK_FILE_ERR     10155   /* File delete error (Kermit) */
#define eDC_MODTIME_FILE_ERR    10156   /* Error setting file modtime */

                                        /* Initialize transfer errors: */
#define eDC_FT_NOT_INIT         10160   /* FTCS structure not init. */
#define eDC_DATABITS_ERR        10161   /* Invalid data bits option */
#define eDC_FLOW_CTRL_ERR       10162   /* Invalid flow control option */

                                        /* Send/receive process errors:*/
#define eDC_TRANSMISSION_ERR    10163   /* Possibly non-fatal status */
#define eDC_INVALID_STATE       10164   /* Internal error or not init. */
#define eDC_EXCEED_MAX_ERRORS   10165   /* Cumulative errs. past limit */
#define eDC_EXCEED_TOUT_RETRIES 10166   /* Too many timeouts */
#define eDC_REMOTE_CANCEL       10167   /* 2 CANs received from remote */

                                        /* Send process errors: */
#define eDC_S_INIT_NAK_TOUT     10168   /* No initial NAK,'C','G', etc */
#define eDC_S_SENDCHAR_TOUT     10169   /* Output queue not draining */
#define eDC_S_WAIT_ACK_TOUT     10170   /* No ACK from receiver */
#define eDC_S_FINAL_ACK_TOUT    10171   /* No final ACK rec'd after EOT */

                                        /* Receive process errors: */
#define eDC_R_INIT_PACK_TOUT    10172   /* No initial packet received */
#define eDC_EXCEED_PACK_RETRIES 10173   /* Exceeded retry limit */
                                        /*   trying to receive a packet */
#define eDC_R_WAIT_SOH_TOUT     10174   /* No incoming packet */
#define eDC_PACK_SEQ_ERR        10175   /* Fatal packet number sequence */
                                        /*   error */
#define eDC_CRC_ERROR           10176   /* Internal error on CRC */

#define eDC_UNEXPECTED_HDR      10177   /* Header unexpected in current */
                                        /*   ZMODEM state */
#define eDC_NAK_RECEIVED        10178   /* Remote received garbled */

#define eDC_Z_WRONG_POSITION    10179   /* Wrong file offset position */
#define eDC_Z_PACKET_SIZE       10180   /* Sender's packet is too large */
#define eDC_R_DISK_FREE         10181   /* Rx not enough room for file */

#define eDC_NO_CARRIER          10190   /* modem signals DCD down */

#define eDC_INVALID_PARAMETER   10191   /* invalid pft initialization */
#define eDC_DECODE_ERROR        10192   /* invalid encode sequence */


/***************************************************************************/
/****** Enumerated types, defines and structures for Dcm port APIs *********/
/***************************************************************************/

/****** Enumerated types used by DcmInit and DcmOpen ***********************/

typedef enum {
        COMPORT1        = 1,
        COMPORT2        = 2,
        COMPORT3        = 3,
        COMPORT4        = 4,
        COMPORT5        = 5,
        COMPORT6        = 6,
        COMPORT7        = 7,
        COMPORT8        = 8
} COMPORTS;

typedef enum {
        BAUD_110        = 110,
        BAUD_150        = 150,
        BAUD_300        = 300,
        BAUD_600        = 600,
        BAUD_1200       = 1200,
        BAUD_2400       = 2400,
        BAUD_4800       = 4800,
        BAUD_9600       = 9600,
        BAUD_19200      = 19200
} BAUDRATES;

typedef enum {
        PARITY_NONE     = 0,
        PARITY_ODD      = 1,
        PARITY_EVEN     = 2
} PARITYS;

typedef enum {
        DATABITS_5      = 5,
        DATABITS_6      = 6,
        DATABITS_7      = 7,
        DATABITS_8      = 8
} DATABITS;

typedef enum {
        STOPBITS_1      = 1,
        STOPBITS_2      = 2
} STOPBITS;


/****** Enumerated types used in other elements of DCMPOPTS structure ******/

typedef enum {
        REMFLOWCTL_DEF  = 0,    /* Remote sw flow control off by default */
        REMFLOWCTL_OFF  = 0,
        REMFLOWCTL_ON   = 1
} REMFLOWCTL;

typedef enum {
        LOCFLOWCTL_DEF  = 0,    /* Local sw flow control off by default */
        LOCFLOWCTL_OFF  = 0,
        LOCFLOWCTL_ON   = 1
} LOCFLOWCTL;

typedef enum {
        BIT7TRIM_DEF    = 0,    /* Bit 7 trimming off by default */
        BIT7TRIM_OFF    = 0,
        BIT7TRIM_ON     = 1
} BIT7TRIM;

typedef enum {
        BIT7FORCE_DEF   = 0,    /* Bit 7 forcing off by default */
        BIT7FORCE_OFF   = 0,
        BIT7FORCE_ON    = 1
} BIT7FORCE;

typedef enum {
        DEFAULT_CTS     = 1,    /* Default - CTS required to transmit */
        IGNORE_CTS      = 0,
        REQUIRE_CTS     = 1
} HWFLOWCTS;


/****** Enumerated type used by DclChgRTS_DTRForOpen ***********************/

typedef enum {
        DEFAULT_RTS_DTR = 0,    /* Default - RTS & DTR are enabled on open */
        ENABLE_RTS_DTR  = 0,
        LEAVE_RTS_DTR   = 1
} OPEN_RTS_DTR;


/****** Enumerated type used by DcmDiscardQPort ****************************/

typedef enum {
        INPUT_Q = 0,
        OUTPUT_Q = 1
} QUEUES;

/****** Enumerated types used by DcmSetHwFlowCtl ***************************/

typedef enum {
        DEFAULT_RTS     = 1,    /* Default - RTS enabled for loc flow ctrl */
        NO_ALTER_RTS    = 0,
        USE_RTS         = 1
} HWFLOWRTS;

typedef enum {
        DEFAULT_DTR     = 1,    /* Default - DTR enabled for loc flow ctrl */
        NO_ALTER_DTR    = 0,
        USE_DTR         = 1
} HWFLOWDTR;


/****** Port Options structure used by the DCMPORT  ************************/

typedef struct                         
{
        BAUDRATES       baud_rate;
        PARITYS         parity;
        DATABITS        data_bits;
        STOPBITS        stop_bits;
        REMFLOWCTL      remote_flow_ctrl;
        LOCFLOWCTL      local_flow_ctrl;
        BIT7TRIM        bit_trimming;
        BIT7FORCE       bit_forcing;
        HWFLOWCTS       require_cts;
        INT             break_time;
} DCMPOPTS;


/****** Port control structure used by the Dcm APIs ************************/

typedef struct {
        COMPORTS        port_no;
        DCMPOPTS        PortOptions;
        UINT            SzInpQ;
        UINT            SzOutQ;
        OPEN_RTS_DTR    enRtsDtr;
} DCMPORT;


/****** Handle to a port control structure used by the Dcm APIs ************/

typedef DCMPORT*        HDCMPORT;


/***************************************************************************/
/****** Enumerated types, defines and structures for Dcm modem APIs ********/
/***************************************************************************/

#define DC_ST_STK_SIZE  8       /* Size of modem state stack   */

#define DC_CMD_LEN      8       /* Maximum command len + null  */
#define DC_CMDSTR_LEN   42      /* Maximum command str length  */
#define DC_RESPSTR_LEN  242     /* Maximum response str length */


/****** Dcm modem options **************************************************/

#define DC_HO_NOOPTION     0x0000       /* Default void option         */
#define DC_HO_ENABLE_REL   0x0001       /* Reliable link option        */
#define DC_HO_ENABLE_COMP  0x0002       /* Compression option for      */
#define DC_HO_FACTORY      0x0004       /* Use factory profile         */
#define DC_HO_SETBAUD      0x0008       /* Tune baud rate              */
#define DC_HO_STOPONERROR  0x0010       /* Stop auto-answer on error   */


/****** Dcm modem response vocabulary **************************************/

struct dcresp_entry {
        struct  dcresp_entry    *pprev; /* Link to previous (NIL to stop)  */
        int                     flags;  /* Various purpose bit flags       */
        char                    *ptxt;  /* Text of modem response          */
        int                     bin;    /* Digital response value          */
        unsigned long           baud_to_use;
                                        /* Baud rate to use after connect  */
};

#define DC_HM_VALID_SIG 0x4d44          /* MODEM signature (MD)            */

typedef enum                            /* Hayes modem models detected:    */
{
    DC_MOD_UNKNOWN,                     /* Not initialized                 */
    DC_MOD_300,                         /* Smartmodem 300                  */
    DC_MOD_1200,                        /* Smartmodem 1200                 */
    DC_MOD_1200PLUS,                    /* Smartmodem 1200B                */
    DC_MOD_2400,                        /* Smartmodem 2400/B/P V series 24 */
    DC_MOD_9600,                        /* Smartmodem 9600/B/V etc.        */
    DC_MOD_14400                        /* ULTRA 14400                     */
} DC_H_MODEL;

typedef enum                            /* Hayes-compatible modem states:  */
{
    DC_H_INIT      = 'i',              /* Start the operation             */
    DC_H_WRITING   = '>',              /* Writing chars to modem          */
    DC_H_PAUSING1  = '1',              /* Pause # 1                       */
    DC_H_PAUSING2  = '2',              /* Pause # 2                       */
    DC_H_PAUSING3  = '3',              /* Pause # 3                       */
    DC_H_WAITRESP  = 'w',              /* Waiting for a modem response    */
    DC_H_QUERYING  = 'q',              /* Querying modem                  */
    DC_H_DRAINING  = 'd',              /* Draining output queue           */
    DC_H_DETECTING = 't',              /* Detecting modem model           */
    DC_H_ENHANCED  = 'e',              /* Sending enhanced commands       */
    DC_H_PREPPARMS = '&',              /* Prepare modem parameters        */
    DC_H_PARMS     = 'p',              /* Sending modem parameters        */
    DC_H_CONFIG    = 's',              /* Send configuration string       */
    DC_H_HANGINGUP = 'h',              /* Hanging up                      */
    DC_H_GOCOMMAND = 'c',              /* Switching to command mode       */
    DC_H_RESETTING = 'r',              /* Resetting modem                 */
    DC_H_SETTING_BD= 'b'               /* Setting baud rate               */
} DCMSTATE;


/****** Modem control structure used by the Dcm APIs ***********************/

typedef struct                         /* Modem definition             */
{
    unsigned  signature;               /* Signature word; set on init  */
    int       port_no;                 /* COM port number              */
    DC_H_MODEL model;                  /* Hayes model detected.        */
    DCMSTATE  state[DC_ST_STK_SIZE];   /* Current modem state stack    */
    int       stkptr;                  /* State stack pointer          */
    long      start_time;              /* Tick count at start of op.   */
    int       outcount;                /* counter of output chars      */
    int       incount;                 /* counter of input chars       */
    unsigned  ib_delay;                /* Inter-byte delay time  (ms)  */
    unsigned  esc_guardtime;           /* Escape guardtime       (ms)  */
    unsigned  resp_delay;              /* Delay before response  (ms)  */
    unsigned  dialtone_wait;           /* Wait for dialtone (S6) (sec) */
    unsigned  dcd_wait;                /* Wait for carrier (S7)  (sec) */
    unsigned  rings_to_answer;         /* Rings until auto-answer (S0) */
    unsigned  query_timeout;           /* Timeout draining out-q (ms)  */
    int       max_query_retries;       /* Maximum retries used by query*/
    int       query_retries;           /* Retries used by query_       */
    int       resp_retries;            /* Retries used by resp_        */
    int       reset_retries;           /* Retries used by reset        */
    unsigned long baud_to_try[6];      /* Baud rates tried in reset_md */
    unsigned long current_baud;        /* Baud rate used by everybody  */
    unsigned long init_baud;           /* Baud rate used by init_md    */
    int       ansr_resp;               /* Response received by [a]ansr */
    int       ansr_error;              /* Error detected by [a]ansr_md */
    int       cwait_resp;              /* Response received by cwait_md*/
    int       cwait_error;             /* Error detected by cwait_md   */
    int       options;                 /* Reliable, Compression        */
    int       save_cmd_dtr;            /* Keeps DTR while commanding   */
    char      escape_char;             /* Escape character (S2)        */
    char      reserved_1;              /* Reserved for future use      */
    char      cmd_prefix[DC_CMD_LEN+1];   /* Command prefix               */
    char      cmd_suffix[DC_CMD_LEN+1];   /* Command suffix               */
    char      dial_prefix[DC_CMD_LEN+1];  /* Dial command and modifiers   */
    char      dial_chars[DC_CMDSTR_LEN+1];/* Valid dial cmd characters    */
    char      hangup[DC_CMD_LEN+1];       /* Hang up phone command (H0)   */
    char      answer[DC_CMD_LEN+1];       /* Answer phone command  (A)    */
    char      get_id[DC_CMD_LEN+1];       /* Get modem id command  (I0)   */
    char      reset[DC_CMD_LEN+1];        /* Reset modem command   (Z)    */
    char      resp_on[DC_CMD_LEN+1];      /* Send response command (Q0)   */
    char      reliable[DC_CMD_LEN+1];     /* Reliable link command (\N3)  */
    char      compression[DC_CMD_LEN+1];  /* Data compression command(%C1)*/
    char      configstr[DC_CMDSTR_LEN+1]; /* Configuration commands       */
    char      cmdstr[DC_CMDSTR_LEN+1];    /* Commands built here          */
    char      respstr[DC_RESPSTR_LEN+1];  /* Response saved here          */
    char      cwrespstr[DC_RESPSTR_LEN+1];/* Carrier response saved here  */
    char      dialstr[DC_CMDSTR_LEN+1];   /* Dial command built here      */
    char      querystr[DC_CMDSTR_LEN+1];  /* Query keeps its string here  */
    char      scratch[DC_CMDSTR_LEN+1];   /* Scratch string for commands  */
    struct    dcresp_entry* presp_list;   /* List of modem responses      */
} DCMMODEM;


/****** Handle to a modem control structure used by the Dcm APIs ***********/

typedef DCMMODEM*       HDCMMODEM;


/***************************************************************************/
/****** Enum, defines and structures for Dcm file transfer APIs ************/
/***************************************************************************/

#define DC_FILE_BUFFER_SIZE     (2048)          /* Size of file I/O buffer */
#ifndef MAXPATH
    #define MAXPATH             (80)            /* Maximum path size */
#endif
#define DC_MAX_PATH_LEN         (MAXPATH)       /* Maximum path length */
#define DC_CANSTRLEN            (21)            /* Cancel string length */
#define DC_ATTNSTRLEN           (32)            /* Attention string length */

typedef enum                    /* Error correction types:             */
{
    DC_NO_ERRCHK = 0,           /* Not initialized                     */
    DC_CHECKSUM  = 1,           /* 8-bit checksum                      */
    DC_CRC       = 2,           /* 16-bit CRC                          */
    DC_CHECKSUM2 = 3,           /* 2-byte checksum for KERMIT          */
    DC_CRC32     = 4            /* 32-bit CRC                          */
} DC_CHKMETHOD;

typedef enum                    /* Supported protocols:                */
{
    DC_UNDEFINED      = 0,
    DC_ASCII          = 1,         /* Straight ASCII                      */
    DC_XMODEM         = 2,         /* XMODEM (Checksum or CRC)            */
    DC_XMODEM_1K      = 3,         /* XMODEM with 1K packets              */
#ifdef  DCM_YMODEM_SUPPORT    
    DC_YMODEM         = 4,         /* YMODEM (Checksum or CRC)            */
    DC_YMODEM_G       = 5,         /* YMODEM-g, full streaming YMODEM     */
#endif  /* DCM_YMODEM_SUPPORT */
    DC_ZMODEM         = 6,         /* ZMODEM protocol                     */
    DC_KERMIT         = 7          /* Loquacious amphibian                */
} DC_PROTOCOL;

typedef enum                    /* Enumerate transfer direction:       */
{
    DC_NO_DIRECTION = 0,
    DC_SENDER       = 1,
    DC_RECEIVER     = 2
} DC_DIRECTION;

typedef enum                    /* ZMODEM header receive states:       */
{
    DC_GETTING_HDR  = 0,           /* Ready to receive a header           */
    DC_WAITING_PAD  = 1,           /* Expecting the PAD character         */
    DC_WAITING_DLE  = 2,           /* Expecting the DLE character         */
    DC_WAITING_IND  = 3,           /* Expecting an indicator              */
    DC_WAITING_DATA = 4            /* Expecting header data               */
} DC_ZHSTATE;

#define DC_FXFER_DONE   1       /* file xfer done */

typedef enum                    /* ZMODEM transfer states:             */
{
    DC_Z_NOT_INIT           = 0 ,  /* Session not initialized             */
    DC_Z_DONE               = 1 ,  /* Session finished                    */
    DC_Z_DRAIN              = 2 ,  /* Draining output queue               */
    DC_ZS_INIT_SEND         = 3 ,  /* Starting the send session           */
    DC_ZR_SND_CHALLENGE     = 4 ,  /* Starting the receive session        */
    DC_Z_FINISH             = 5 ,  /* Ending the session                  */
    DC_Z_CANCEL             = 7 ,  /* Session is to be aborted            */
    DC_ZS_SND_RQINIT        = 11,  /* Send ZRQINIT header                 */
    DC_ZS_GET_PAD           = 12,  /* Wating for PAD characters           */
    DC_ZS_GET_RINIT         = 13,  /* Waiting for ZRINIT header           */
    DC_ZS_GET_SINIT_ACK     = 14,  /* Waiting for ZACK to ZSINIT          */
    DC_ZS_GET_RPOS          = 15,  /* Waiting for ZRPOS header            */
    DC_ZS_SND_CRC           = 16,  /* Send ZCRC header                    */
    DC_ZS_GET_ACK           = 17,  /* Waiting for ZACK header             */
    DC_ZS_SND_PACKET        = 18,  /* Send data subpacket                 */
    DC_ZS_CLOSE             = 19,  /* Waiting for a response to ZEOF      */
    DC_ZS_SND_COMMAND       = 20,  /* Send a ZCOMMAND                     */
    DC_ZS_GET_COMPL         = 21,  /* Waiting for a response to ZCOMMAND  */
    DC_ZS_GET_DFREE_ACK     = 22,  /* Waiting for ZACK to ZDFREE          */
    DC_ZS_GET_FIN_ACK       = 23,  /* Waiting for ZACK to ZFIN            */
    DC_ZR_GET_PAD           = 24,  /* Waiting for PAD characters          */
    DC_ZR_GET_CHALLENGE_ACK = 25,  /* Waiting for ZACK to ZCHALLENGE      */
    DC_ZR_SND_RINIT         = 26,  /* Send a ZRINIT                       */
    DC_ZR_GET_FILE          = 27,  /* Waiting for ZFILE                   */
    DC_ZR_GET_FILE_CRCW     = 28,  /* Waiting for ZFILE data subpacket    */
    DC_ZR_GET_SINIT_CRCW    = 29,  /* Waiting for ZSINIT data subpacket   */
    DC_ZR_GET_CRC           = 30,  /* Waiting for ZCRC                    */
    DC_ZR_GET_PACKET        = 31,  /* Waiting for a data packet           */
    DC_ZR_GET_EOF           = 32,  /* Waiting for a ZEOF                  */
    DC_ZR_SND_ATTN          = 33,  /* Send an attention sequence          */
    DC_ZR_GET_DATA          = 34,  /* Receiving data                      */
    DC_ZR_GET_COMMAND       = 35,  /* Waiting for ZCOMMAND                */
    DC_ZR_SND_ZCOMPL        = 36   /* Sending acknowledge                 */
} DC_ZMSTATE;


/****** File transfer structures for Dcm file transfer APIs ****************/

typedef struct                          /* DOS Disk Transfer Area      */
{
    char _reserved[21];                 /* Reserved by DOS             */
    char _attribute;                    /* File attribute              */
    int  _filetime;                     /* File time                   */
    int  _filedate;                     /* File date                   */
    long _filesize;                     /* File size                   */
    char _filename[13];                 /* File name (ASCIIZ)          */
    char _pad_char;                     /* Word alignment              */

} DC_F_BLOCK;


/****** File transfer structure used by the Dcm APIs ***********************/

typedef struct                          /* File transmission parameters*/
{
    unsigned    signature;              /* Signature word; set on init */
    unsigned    port_no;                /* COM port number             */
#ifdef  DC_USING_LEVEL_0
    const volatile COM_DEF far *port;   /* COM port data structure     */
#else    
    const volatile void    far *port;   /* COM port data structure     */
#endif  /* DC_USING_LEVEL_0 */
    DC_PROTOCOL protocol;               /* Protocol X/YZMODEM, KERMIT  */
    DC_DIRECTION direction;             /* Sender? Receiver?           */
    int         cur_state;              /* Current transfer state      */
    int         next_state;             /* Next transfer state         */
    int         cur_error;              /* Last error reported         */
    int         fatal_error;            /* Error causing abort transfer*/
    DC_CHKMETHOD errchk_method;         /* CHECKSUM, CHECKSUM2, CRC[32]*/
    int         x_scratch;              /* X\YMODEM\ASCII: Rcvd. CANs  */
    int         y_scratch;              /* Scratch field for yfer_ft() */
    int         z_scratch;              /* Scratch field for zfer_ft() */

    /* Transfer options -----------------------------------------------*/
    unsigned    options;                /* General option bits         */
    unsigned    zr_options;             /* ZMODEM receiver options     */
    unsigned    zc_option;              /* ZMODEM conversion option    */
    unsigned    zm_option;              /* ZMODEM management option    */

    /* Time -----------------------------------------------------------*/
    long        start_time;             /* Tick count at start of op.  */
    int         relax_factor;           /* Timeout relaxation factor   */

    /* Maximum retries values and retries counters --------------------*/
    int         max_packet_retries;     /* Max retries to send packet  */
    int         cur_packet_retries;     /* Current packet retry count  */
    int         max_tout_retries;       /* Max timeouts in the packet  */
    int         cur_tout_retries;       /* Current timeout retry count */
    int         max_cum_retries;        /* Cumulative retries allowed  */
    int         cum_total_retries;      /* Current cumulative retry cnt*/
    int         max_startup_retries;    /* Maximum value               */
                                        /* for cur_pack_retries before */
                                        /* contact with the other side */
                                        /* is established              */
    int         max_crc_retries;        /* (XMODEM only) Maximum value */
                                        /* for cur_pack_retries before */
                                        /* switching to checksum       */

    /* Timeouts -------------------------------------------------------*/
    int         r_soh_tout;             /* Timeout waiting for SOH     */
    int         s_soh_tout;             /* Timeout waiting for SOH     */
    int         r_trycrc_tout;          /* Timeout initiating CRC xfer */
    int         r_inchar_tout;          /* Timeout between characters  */
    int         s_outchar_tout;         /* Timeout between characters  */
    int         s_startup_tout;         /* Timeout waiting for init NAK*/
    int         s_ack_tout;             /* Timeout waiting for ACK     */
    int         header_tout;            /* Timeout waiting for header  */
    int         packet_tout;            /* Timeout waiting for packet  */

    /* Data parameters ------------------------------------------------*/
    long        packet_count;           /* Total packets transmitted   */
    int         packet_no;              /* Current packet # (mod 256)  */
                                        /* (mod 64 for KERMIT)         */
    int         small_data_size;        /* Small packet size in bytes  */
    int         large_data_size;        /* Large packet size in bytes  */
    int         cur_data_size;          /* Total bytes in data/window  */
    long        cur_pack_size;          /* Total bytes in a packet     */
    int         window_size;            /* (KERMIT or ZMODEM)          */
                                        /* Data window size            */
    int         cur_window_size;        /* (KERMIT or ZMODEM)          */
                                        /* Current data window size    */

    /* ADJUST_FT fields -----------------------------------------------*/
    int         cum_resend;             /* Cumulative resends          */
    int         cum_sent;               /* Cum. bytes without resend   */

    /* File parameters ------------------------------------------------*/
    const char *ppath;                  /* File path(s)                */
    char cur_path [DC_MAX_PATH_LEN];    /* Current file path name      */
    FILE       *pfile;                  /* Pointer to file variable    */
    DC_F_BLOCK  fblock;                 /* DTA block for multi-file    */
    time_t      fmodtime;               /* File modification time      */
    int         num_files;              /* Approx. number files to send*/
    int         file_count;             /* Files transmitted           */
    long        fsize;                  /* File size                   */
    long        cur_fsize;              /* File size/bytes transmitted */
    long        packets_in_file;        /* Total packets required      */

    /* Buffer for data packets ----------------------------------------*/
    unsigned char  *packet_buffer;      /* (KERMIT only) Buffer pool   */
                                        /* for data packets or single  */
                                        /* buffer, if no window        */
    unsigned char  *ppacket;            /* Pointer to packet buffer    */
    int             packet_index;       /* Offset of current character */

    /* File I/O -------------------------------------------------------*/
    unsigned char fbuffer [DC_FILE_BUFFER_SIZE]; /* file i/o buffer    */
    int             fbuffer_size;       /* Bytes in file buffer.       */
    int             fbuffer_index;      /* Bytes read from file buffer */

    /* Encoding & decoding --------------------------------------------*/
    int             last_char;          /* Last char (for TRANSLATE_NL)*/
    int             status;             /* Encoding status.            */
    unsigned char   tail [4];           /* Escape sequence that doesnt */
                                        /* fit the current packet      */
    int             tail_size;          /* Number of bytes in tail     */

    /* Cancellation & Errors ------------------------------------------*/
    int cum_status_errors;              /* Cumulative status errors    */
    int cancel_trans;                   /* User cancel transfer flag   */
    unsigned char cancel_str [DC_CANSTRLEN]; /* Cancel string          */

    /* ZMODEM structure fields ----------------------------------------*/
    int             cur_zcrcstate;      /* State of CRC process(ZMODEM)*/
    unsigned char   checksum;           /* Checksum error packet       */
    unsigned int    crc16;              /* CRC-16 error packet         */
    unsigned long   crc32;              /* CRC-32 error packet         */
    DC_ZHSTATE      cur_zhstate;        /* State of header receive     */
    long            l_scratch;          /* Scratch field for zfer_ft() */
    long            append_offset;      /* File offset for APPEND ops  */
    int             cur_hdr_size;       /* Size of the header received */
    unsigned char   attn_str[DC_ATTNSTRLEN]; /* Attention string       */
    unsigned char   cur_data_type;      /* CRCW/CRCG/CRCQ/CRCE,        */
    unsigned char   data_type;          /* CRCW/CRCG/CRCQ              */
    unsigned char   init_data_type;     /* CRCW/CRCG/CRCQ, session     */

    /* KERMIT structure fields ----------------------------------------*/
    int             k_small_data_size;  /* Maximum size of small packet*/
    int             k_npad;             /* Number of padding characters*/
    unsigned char   padding_char;       /* Padding character           */
    unsigned char   eol_char;           /* Packet terminator           */
    unsigned char   qctl_char;          /* Char for quoting controls   */
    unsigned char   qbin_char;          /* Char for quoting not ASCII  */
    unsigned char   rept_char;          /* Char for quoting repetitions*/
    char            word_align_byte;

    /* Logging --------------------------------------------------------*/
    FILE      *plogfile;                /* Debug logging file          */

} DCMFXFER;

/****** Handle to a file transfer structure used by the Dcm APIs ***********/

typedef DCMFXFER*       HDCMFXFER;


/***************************************************************************/
/****** Dcm API prototype definitions **************************************/
/***************************************************************************/

#ifndef __IN_PENRIGHT_H
#ifdef  __cplusplus
extern "C"
{
#endif  /* __cplusplus */
#endif  /* __IN_PENRIGHT_H */ 

#ifdef  DEFININGDCM
#define DECLARE
#else
#define DECLARE extern
#endif

DECLARE INT     PRDCAPI         DcmInitPort(
                                HDCMPORT        hDcmPort,
                                COMPORTS        enPortNo);

DECLARE INT     PRDCAPI         DcmOpenPort(
                                HDCMPORT        hDcmPort,
                                BAUDRATES       enBaudRate,
                                PARITYS         enParity,
                                DATABITS        enDataBits,
                                STOPBITS        enStopBits);

DECLARE INT     PRDCAPI         DcmClosePort(
                                HDCMPORT        hDcmPort);

DECLARE INT     PRDCAPI         DcmGetPortParams(
                                HDCMPORT       hDcmPort);

DECLARE INT     PRDCAPI         DcmSetPortParams(
                                HDCMPORT       hDcmPort);

DECLARE INT     PRDCAPI         DcmSetSwFlowCtl(
                                HDCMPORT        hDcmPort,
                                REMFLOWCTL      enFlowRem,
                                LOCFLOWCTL      enFlowLoc);

DECLARE INT     PRDCAPI         DcmSetHwFlowCtl(
                                HDCMPORT        hDcmPort,
                                HWFLOWCTS       enFlowCts,
                                HWFLOWRTS       enFlowRts,
                                HWFLOWDTR       enFlowDtr);

DECLARE INT     PRDCAPI         DcmWritePort(
                                HDCMPORT        hDcmPort,
                                INT             nbytes,
                                LPCSTR          pBuffer,
                                LPINT           pNumWritten);

DECLARE INT     PRDCAPI         DcmReadPort(
                                HDCMPORT        hDcmPort,
                                INT             nbytes,
                                LPCSTR          pBuffer,
                                LPINT           pNumRead,
                                LPINT           pNumLeft);

DECLARE INT     PRDCAPI         DcmDrainPort(
                                HDCMPORT        hDcmPort,
                                LONG            timeout,
                                P_CAN_FUNC      pCancelFunc);

DECLARE INT     PRDCAPI         DcmDiscardQPort(
                                HDCMPORT        hDcmPort,
                                QUEUES          enQueue);


DECLARE INT     PRDCAPI         DcmInitModem(
                                HDCMMODEM*      phDcmModem,
                                HDCMPORT        hDcmPort,
                                LPDWORD         pBaudCode,
                                LPINT           pRespCode,
                                BOOL            tPowerOnDefaults);

DECLARE INT     PRDCAPI         DcmDialModem(
                                HDCMMODEM       hDcmModem,
                                LPCSTR          pDialString,
                                LPINT           pRespCode,
                                P_CAN_FUNC      pCancelFunc);

DECLARE INT     PRDCAPI         DcmAAnsrModem(
                                HDCMMODEM       hDcmModem,
                                UINT            num_rings,
                                LPDWORD         pNewBaud,
                                LPINT           pRespCode,
                                P_CAN_FUNC      pCancelFunc);

DECLARE INT     PRDCAPI         DcmHangUpModem(
                                HDCMMODEM       hDcmModem);

DECLARE INT     PRDCAPI         DcmInitFXfer(
                                HDCMFXFER*      phDcmFXfer,
                                HDCMPORT        hDcmPort,
                                DC_DIRECTION    enDirection,
                                LPCSTR          pFilePath,
                                DC_PROTOCOL     enProtocol,
                                DC_CHKMETHOD    enErrChk,
                                INT             nRetries);


DECLARE INT     PRDCAPI         DcmFXferAscii(
                                HDCMFXFER       hDcmFXfer,
                                P_CAN_FUNC      pCancelFunc);
                                
DECLARE INT     PRDCAPI         DcmFXferXmodem(
                                HDCMFXFER       hDcmFXfer,
                                P_CAN_FUNC      pCancelFunc);

DECLARE INT     PRDCAPI         DcmFXferZmodem(
                                HDCMFXFER       hDcmFXfer,
                                P_CAN_FUNC      pCancelFunc);

DECLARE INT     PRDCAPI         DcmFXferKermit(
                                HDCMFXFER       hDcmFXfer,
                                P_CAN_FUNC      pCancelFunc);

DECLARE INT     PRDCAPI         DcmSetInpQSize(
                                HDCMPORT        hDcmPort,
                                INT             nInpQLength);

DECLARE INT     PRDCAPI         DcmSetOutQSize(
                                HDCMPORT        hDcmPort,
                                INT             nOutQLength);

DECLARE VOID    PRDCAPI         DcmChgRTS_DTRForOpen(
                                HDCMPORT        hDcmPort,
                                OPEN_RTS_DTR    enRTS_DTR);

DECLARE INT     PRDCAPI         DcmInpQSize(
                                HDCMPORT        hDcmPort,
                                LPINT           QSize);

#undef  DECLARE /* DEFININGDCM */


/***************************************************************************/
/****** Dcl API prototype definitions **************************************/
/***************************************************************************/

#ifdef  DEFININGDCL
#define DECLARE
#else
#define DECLARE extern
#endif  /* DEFININGDCL */

DECLARE INT     PRDCAPI         DclCmdRspModem(
                                HDCMMODEM       hDcmModem,
                                LPCSTR          pCommand,
                                LPSTR           pResponse,
                                LPINT           pRespCode,
                                LPINT           pErrCode);

DECLARE INT     PRDCAPI         DclSendBreak(
                                HDCMPORT    hDcmPort);

DECLARE INT     PRDCAPI         DclInDclCritical(
                                HDCMPORT        hDcmPort,
                                LPINT           pBusyFlag,
                                LPINT*          pBFlagPtr);

#undef  DECLARE /* DEFININGDCL */


#ifndef __IN_PENRIGHT_H
#ifdef  __cplusplus
}
#endif  /* __cplusplus */
#endif  /* __IN_PENRIGHT_H */

#endif  /* __DCMAPI */          

