/****************************************************************************
*
*	Module Name: gpddgt.h
*
*	Description:
*
*	Revision History:
*	4.0	11/23/94		FJL
*			Made the following changes:
*			1.	Added conditional compilation of prototype for DgtInitialize().
*				Identifier "PRWIN" is used for Window's compilation.
*
****************************************************************************/

/*
 * $Workfile:   GPDDGT.H  $
 *
 * Current $Revision:   1.1  $ By $Author:   westerinen  $
 *
 * $Modtime:   03 Dec 1992 17:40:52  $
 *
 * Reverse Chronological Revision 
 *
 * $Log:   H:/penright/vcs/incs/gpddgt.h_v  $
 * 
 *    Rev 1.1   03 Dec 1992 17:40:58   westerinen
 * added vcs header
 *
*/

/*--------------------------------------------------------*/
/*                Digitizer include file                  */
/*--------------------------------------------------------*/

/****************************************************************
 This include file uses the defined symbols DGTEXTSTATUSASBITS, 
 DGTSTATEASBITS and DGTCAPABILITYASBITS to provide alternative ways 
 to access the values encoded in DGTEXTSTATUSTYPE.status, 
 DGTPOINTPACKETTYPE.state and DGTDEVICETYPE.capability respectively.
 
 When one of these symbols is defined the corresponding structure 
 element is interpreted as a bit value and the members of DGTEXTSTATUSBITS, 
 DGTSTATEBITS or DGTCAPABILITYBITS should be used to access the element
 rathar than the defines DGTPENBATTERYLOW..DGTPOINTBUFFERFULL, 
 DGTBTNONEPRESSED..DGTINPROXIMITY or DGTPROXIMITY..DGTBUFFERING.

 These symbols are normally defined by setting a command line option
 for the C compiler: -DDGTEXTSTATUSASBITS, -DDGTSTATEASBITS or 
 -DDGTCAPABILITYASBITS
****************************************************************/


/* access DGTPOINTPACKETTYPE.state as an unsigned short value           */
/* masks for state info                       bits used                 */
#define DGTBTNONEPRESSED         1   /* - - - - - - - - - - - - - - - b */
#define DGTBTNTWOPRESSED         2   /* - - - - - - - - - - - - - - b - */
#define DGTBTNTHREEPRESSED       4   /* - - - - - - - - - - - - - b - - */
#define DGTBUFFEROVERFLOWED      8   /* - - - - - - - - - - - - b - - - */
#define DGTINPROXIMITY       32768U  /* b - - - - - - - - - - - - - - - */

/* access DGTPOINTPACKETTYPE.state as a bit field value                 */
typedef struct
{
  unsigned  btnOnePressed:    1;
  unsigned  btnTwoPressed:    1;
  unsigned  btnThreePressed:  1;
  unsigned  bufferOverflowed: 1;
  unsigned  reservedFlags:    2;
  unsigned  oemFlags:         9;
  unsigned  inProximity:      1;
} DGTSTATEBITS;

/* access DGTDEVICETYPE.capability as an unsigned short value           */
/* masks for capability info                  bits used                 */
#define DGTPROXIMITY             1   /* - - - - - - - - - - - - - - - b */
#define DGTTWOBUTTONS            2   /* - - - - - - - - - - - - - - b - */
#define DGTTHREEBUTTONS          4   /* - - - - - - - - - - - - - b - - */
#define DGTLOWPOWER              8   /* - - - - - - - - - - - - b - - - */
#define DGTHEIGHT               16   /* - - - - - - - - - - - b - - - - */
#define DGTPRESSURE             32   /* - - - - - - - - - - b - - - - - */
#define DGTBUFFERING            64   /* - - - - - - - - - b - - - - - - */

/* access DGTDEVICETYPE.capability as a bit field value                 */
typedef struct
{
  unsigned  proximity:     1;
  unsigned  twoButtons:    1;
  unsigned  threeButtons:  1;
  unsigned  lowPower:      1;
  unsigned  height:        1;
  unsigned  pressure:      1;
  unsigned  buffering:     1;
} DGTCAPABILITYBITS;

/* access DGTEXTSTATUSTYPE.status as an unsigned short value            */
/* masks for status info                      bits used                 */
#define DGTPENBATTERYLOW         1   /* - - - - - - - - - - - - - - - b */
#define DGTSTATUSBIT0            2   /* - - - - - - - - - - - - - - b - */
#define DGTSTATUSBIT1            4   /* - - - - - - - - - - - - - b - - */
#define DGTSTATUSBIT2            8   /* - - - - - - - - - - - - b - - - */
#define DGTPOINTBUFFERFULL      16   /* - - - - - - - - - - - b - - - - */

/* access DGTEXTSTATUSTYPE.status as a bit field value                  */
typedef struct
{
  unsigned  penBatteryLow:   1;
  unsigned  statusBit0:      1;
  unsigned  statusBit1:      1;
  unsigned  statusBit2:      1;
  unsigned  pointBufferFull: 1;
} DGTEXTSTATUSBITS;

typedef struct
{
  unsigned short    minx;
  unsigned short    miny;
  unsigned short    maxx;
  unsigned short    maxy;
} DGTEXTENTTYPE;

typedef struct
{
  unsigned short    x;
  unsigned short    y;
#if defined(DGTSTATEASBITS)
  DGTSTATEBITS      state;
#else
  unsigned short    state;
#endif
  void far         *oemPointData;
} DGTPOINTPACKETTYPE;

typedef struct
{
  unsigned short    id;
  unsigned short    ver;
  unsigned short    prCompatibility;
#if defined(DGTCAPABILITYASBITS)
  DGTCAPABILITYBITS capability;
#else
  unsigned short    capability;
#endif
  void far         *oemDeviceData;
  char far         *deviceName;
} DGTDEVICETYPE;

typedef struct
{
#if defined(DGTEXTSTATUSASBITS)
  DGTEXTSTATUSBITS  status;
#else
  unsigned short    status;
#endif
  void far         *oemExtendedStatus;
} DGTEXTSTATUSTYPE;


#ifdef APIINTERNALCALLER
#define CALLCONVENTION cdecl
#else
#define CALLCONVENTION pascal      
#endif

#ifdef PRWIN
extern unsigned short CALLCONVENTION DgtInitialize (HWND hWnd);
#else
extern unsigned short CALLCONVENTION DgtInitialize (void);
#endif

extern unsigned short CALLCONVENTION DgtTerminate (void);


#ifdef PRWIN
extern void CALLCONVENTION DgtSetFocusFlag(BOOLEAN value);
extern void CALLCONVENTION DgtSetCursor(BOOLEAN value);
extern void CALLCONVENTION DgtFlushBuffer(void);
#endif

extern unsigned short CALLCONVENTION DgtGetExtents 
  (DGTEXTENTTYPE far *extents);

extern unsigned short CALLCONVENTION DgtGetPoint 
  (DGTPOINTPACKETTYPE far *packet);

extern unsigned short CALLCONVENTION DgtGetResolution 
  (unsigned short far *resolution);

extern unsigned short CALLCONVENTION DgtGetDeviceType 
  (DGTDEVICETYPE far *device);

extern unsigned short CALLCONVENTION DgtGetSamplingRate 
  (unsigned short far *rate);

extern unsigned short CALLCONVENTION DgtSetSamplingRate 
  (unsigned short rate);

extern unsigned short CALLCONVENTION DgtGetThreshold 
  (unsigned short far *threshold);

extern unsigned short CALLCONVENTION DgtSetThreshold 
  (unsigned short threshold);

extern unsigned short CALLCONVENTION DgtGetBuffering 
  (unsigned short far *value);

extern unsigned short CALLCONVENTION DgtSetBuffering 
  (unsigned short value);

extern unsigned short CALLCONVENTION DgtConfigure 
  (void far *configInfo);

extern unsigned short CALLCONVENTION DgtExtendedStatus 
  (DGTEXTSTATUSTYPE far *statusInfo);

extern unsigned short CALLCONVENTION DgtRegisterEvents 
  (void (cdecl *CallBackFcn) (void), unsigned short far *eMask);

extern CDRINFOTYPE far * CALLCONVENTION DgtGetCdrInfoPtr (void);
