/****************************************************************************
*
*	Module Name: gpderr.h
*
*	Description:
*
*	Revision History:
*	4.0	11/09/94		FJL
*			Made the following changes:
*			1.	Added new error, eMultipleInstance. This error is returned
*				by ApiInitialize() if another application is already running.
*
*	4.0	11/23/94		FJL
*			Made the following changes:
*			1.	Added conditional compilation for definition of error constant
*				eMultipleInstance. Indentifier "PRWIN" is used for Window's
*				compilation.
*
*	4.0	03/21/95		ECM
*			Made the following changes:
*			1.	Added eFontResource to the list of error constants.
*
*	0.8	05/15/95		ECM
*			Made the following changes:
*			1.	Added eSystemResource to the list of error constants.
*
****************************************************************************/

/*
 * $Workfile:   gpderr.h  $
 *
 * Current $Revision:   1.9  $ By $Author:   westerinen  $
 *
 * $Modtime:   24 May 1994 16:23:04  $
 *
 * Reverse Chronological Revision 
 *
 * $Log:   H:\penright\vcs\incs\gpderr.h_v  $
 * 
 *    Rev 1.9   24 May 1994 16:26:32   westerinen
 * 
 * Added error eKbdInvalidObjectSeqIndex.
 * 
 *    Rev 1.8   24 May 1994 12:53:24   westerinen
 * 
 * Added eKbdNavNotInstalled error.
 * 
 *    Rev 1.7   17 May 1994 09:57:58   rahman
 * PR3.6: Added eKbdCurrentObjIndex error code.
 * 
 *    Rev 1.6   04 May 1994 13:23:56   westerinen
 * Added eKbdNonVanObj.
 * 
 *    Rev 1.5   03 May 1994 12:29:58   westerinen
 * Changed eNoActiveForm to eKbdNoCurrentForm.
 * 
 *    Rev 1.4   26 Apr 1994 16:59:40   westerinen
 * 
 * Added keyboard navigation errors.
 * 
 *    Rev 1.3   17 Jun 1993 10:45:12   westerinen
 * Added five more Overlay.. error definitions to complete error set returned 
 * from pen driver calls from the Ovl code.
 * 
 *    Rev 1.2   05 May 1993 17:16:58   westerinen
 * Fix for PAR 30429009.  Removed error 9800. Added error 9050. Updated error 
 * range definitions.
 * 
 *    Rev 1.1   03 Dec 1992 17:43:08   westerinen
 * added vcs header
 *
*/


/*--------------------------------------------------------*/
/*               Error codes for PenRight                 */
/*--------------------------------------------------------*/

/*  Msdos error codes  */

#define eMsOK                      0
#define eMsInvalidParm             1
#define eMsFileNotFound            2
#define eMsPathNotFound            3
#define eMsTooManyOpen             4
#define eMsAccessDenied            5
#define eMsInvalidConn             6
#define eMsMemoryTrashed           7
#define eMsOutOfMemory             8
#define eMsInvalidBlock            9
#define eMsBadEnvironment         10
#define eMsBadFormat              11
#define eMsInvalidAccess          12
#define eMsInvalidData            13

#define eMsInvalidDrive           15
#define eMsDirectoryInUse         16
#define eMsNotSameDevice          17
#define eMsNoMoreFiles            18

/* Error codes for init routines */

#define eLibraryNotLoaded       9000
#define eIncompatibleAPI        9001
#define eIncompatibleHWP        9002
#ifdef PRWIN
#define eMultipleInstance       9003
#define eFontResource			  9004
#define eSystemResource			  9005
#endif

/* Error codes for window routines */

#define eInvalidWindowID        9010
#define eInvalidWindowFormat    9011
#define eInvalidWindowSize      9012

/* Error codes for resource routines */

#define eTooManyResourceConns   9020
#define eInvalidResourceFile    9021
#define eResourceNotFound       9022
#define eInvalidCanvasResource  9023
#define eResourceFileClosed     9024 

/* Error codes for event routines */

#define eEventQueueIsFull       9030

/* Error codes for font routines */

#define eInvalidFont            9040

/* Error codes for runtime/pendriver interaction */

#define eIncompatiblePenDriver  9050


/* Error codes for keyboard navigation routines */

#define eKbdFirstObject             9060
#define eKbdFormObjSeqNotFound      9061
#define eKbdInvalidFormId           9062
#define eKbdInvalidFormObjSeqPtr    9063
#define eKbdInvalidObjectId         9064
#define eKbdInvalidObjectIndex      9065
#define eKbdInvalidObjectType       9066
#define eKbdLastObject              9067
#define eKbdNavigationDisabled      9068
#define eKbdObjectFocusDisabled     9069
#define eKbdObjectNotFound          9070
#define eKbdObjectSeqFull           9071
#define eKbdNoMarquee               9072
#define eKbdNoCurrentForm           9073
#define eKbdNonNavObj               9074
#define eKbdNoCurrentObjIndex       9075
#define eKbdNavNotInstalled         9076
#define eKbdInvalidObjectSeqIndex   9077

/* Error codes for overlay routines */

#define eOverlayNotActive        9100
#define eOverlayRangeError       9101
#define eOverlayInitError        9102
#define eOverlayDataNotAvailable 9103
#define eOverlayInvalidFcn       9104
#define eOverlayTimeout          9105

/* Error codes for handwriting routines */

/* Error codes for field routines */

#define eFieldTooLarge          9401
#define eFieldFull              9402
#define eScrapTooSmall          9403
#define eInvalidNumericValue    9404
#define eInvalidFormat          9405
#define eNumericOverflow        9406

/* Error codes for form routines */

#define eFontNotLoaded          9450
#define eInvalidParam           9451

/* Error codes for resource building */

/* Error codes for form building */

/* Error codes from pen driver */
#define ePenDrvrGeneralError   37120
#define ePenDrvrRangeError     37121
#define ePenDrvrInitError      37122
#define ePenDrvrDataNotAvail   37123
#define ePenDrvrInvalidFcn     37124
#define ePenDrvrTimeout        37125

#define ePenDrvrNotSupported   37254


/****************************************************************************
*
*   Error range definitions
*
*   9000 API Kernel
*   9050  "    "
*   9100 Overlay (not valid for PenRight! ver 3.1 or greater)
*   9150 Handwriting
*   9200 Database
*   9250    "
*   9300
*   9350
*   9400 API Fields and Forms
*   9450  "    "
*   9500 Resource Builder
*   9550
*   9600 Form Builder
*   9650  "    "
*   9700 CodeStrt
*   9750
*   9800 DsnForm
*   9850
*   9900 PenRes
*   9950
*  10000 Datacomm 
*  10100    "
*  10200
*  20000
*  30000
*  37120 PenDriver
*  
****************************************************************************/
