/****************************************************************************
*
*	Module Name: gpdevt.h
*
*	Description:
*
*	Revision History:
*	0.6	02/10/95		FJL
*			Replaced "pascal" in all API function prototypes with _APICALL
*			which is defined as "pascal far" for Windows and "pascal" for DOS.
*
****************************************************************************/

/*
 * $Workfile:   GPDEVT.H  $
 *
 * Current $Revision:   1.1  $ By $Author:   westerinen  $
 *
 * $Modtime:   03 Dec 1992 17:43:42  $
 *
 * Reverse Chronological Revision 
 *
 * $Log:   H:/penright/vcs/incs/gpdevt.h_v  $
 * 
 *    Rev 1.1   03 Dec 1992 17:43:48   westerinen
 * added vcs header
 *
*/


/*--------------------------------------------------------*/
/*                  Events include file                   */
/*--------------------------------------------------------*/

/* Causes for events */
#define PENDOWN_CAUSE               1
#define HANDWRITING_CAUSE           3 
#define STROKES_CAPTURED_CAUSE      HANDWRITING_CAUSE
#define TYPED_CHAR_CAUSE            4
#define DOUBLE_PENDOWN_CAUSE        5
#define CROSSOUT_CAUSE              6
#define INSERT_BLANK_CAUSE          7
#define GESTURE_RECOGNIZED_CAUSE    8
#define INSERT_CRLF_CAUSE           9

/* Gesture codes */
#define GSTR_SINGLE_TAP             1
#define GSTR_DOUBLE_TAP             2
#define GSTR_PENDOWN_AND_HOLD       3
#define GSTR_V_CROSSOUT             4
#define GSTR_H_CROSSOUT             5
#define GSTR_DELETE                 6
#define GSTR_INSERT_SPACE           7
#define GSTR_INSERT_SPACES          8
#define GSTR_INSERT_CRLF            9

/* Mappings for event data fields */

/* ctlEnterEvent */
#define CTL_ENTER_CONTROL_ID        data1
#define CTL_ENTER_CONTROL           data2

/* ctlExitEvent */
#define CTL_EXIT_CONTROL_ID         data1
#define CTL_EXIT_CONTROL            data2
#define CTL_EXIT_CURR_VALUE         data3
#define CTL_EXIT_ITEM               data4
#define CTL_EXIT_NEXT_VALUE         data5

/* ctlPressedEvent */
#define CTL_PRESSED_CONTROL_ID      data1
#define CTL_PRESSED_CONTROL         data2
#define CTL_PRESSED_CURR_VALUE      data3
#define CTL_PRESSED_ITEM            data4
#define CTL_PRESSED_NEXT_VALUE      data5

/* firstStrokeEvent */
#define FIRST_STRK_PENDOWN_1        data1
#define FIRST_STRK_PENDOWN_2        data2
#define FIRST_STRK_PENUP_1          data3
#define FIRST_STRK_PENUP_2          data4

/* fldChangedEvent */
#define FLD_CHANGE_CURR_FLD_ID      data1
#define FLD_CHANGE_CURR_FLD         data2
#define FLD_CHANGE_CAUSE            data3
#define FLD_CHANGE_HWR_PENDING      data4

/* fldEnterEvent */
#define FLD_ENTER_CURR_FLD_ID       data1
#define FLD_ENTER_CURR_FLD          data2
#define FLD_ENTER_CAUSE             data3
#define FLD_ENTER_PREV_FLD_ID       data4
#define FLD_ENTER_PREV_FLD          data5
#define FLD_ENTER_GESTURE_CODE      data6
#define FLD_ENTER_NEXT_STROKE       data6

/* handwritingRecognizedEvent */
#define HW_REC_FIRST_STROKE         data1
#define HW_REC_LAST_STROKE          data2
#define HW_REC_PTEXT_SEGMENT        data3
#define HW_REC_PTEXT_OFFSET         data4
#define HW_REC_TEXT_LEN             data5

/* keyDownEvent */
#define KEYDOWN_EXTENDED_KEY        data1

/* lstEnterEvent */
#define LST_ENTER_LIST_ID           data1
#define LST_ENTER_LIST              data2
#define LST_ENTER_DOUBLE_DOWN       data3

/* lstChangedEvent */
#define LST_CHANGE_LIST_ID          data1
#define LST_CHANGE_LIST             data2
#define LST_CHANGE_LIST_CHOICE      data3

/* lstExitEvent */
#define LST_EXIT_LIST_ID            data1
#define LST_EXIT_LIST               data2
#define LST_EXIT_LIST_CHOICE        data3

/* penDownEvent */
#define PENDOWN_DOUBLE_DOWN         data1
#define PENDOWN_DOUBLE_DOWN_X       data2
#define PENDOWN_DOUBLE_DOWN_Y       data3

#define DOUBLE_PENDOWN_MASK         1
#define SINGLE_PENDOWN              0
#define DOUBLE_PENDOWN              1

/* saveDisplayEvent */
#define SAVEDISPLAY_PREV_ETYPE      data1
#define SAVEDISPLAY_TARGET_FLD      data2
#define SAVEDISPLAY_ENTER_CAUSE     data3

/* strokesCapturedEvent */
#define STROKE_CAPTURED_TARGET_FLD  data1

/* winExitEvent */
#define WIN_EXIT_CURR_WIN_HANDLE    data1
#define WIN_EXIT_NEXT_WIN_HANDLE    data2

/* winEnterEvent */
#define WIN_ENTER_CURR_WIN_HANDLE   data1
#define WIN_ENTER_PREV_WIN_HANDLE   data2


#define LAST_GPEN_EVENT   32767

enum events 
{
  nullEvent = 0,
  penDownEvent,
  penUpEvent,
  reservedEvent1,
  keyDownEvent,
  winEnterEvent,
  winExitEvent,
  menuEvent,
  ctlEnterEvent,
  ctlExitEvent,
  lstEnterEvent,
  lstChangedEvent,
  lstExitEvent,
  fldEnterEvent,
  fldChangedEvent,
  ctlPressedEvent,
  firstStrokeEvent,
  strokesCapturedEvent,
  handwritingRecognizedEvent,
  menuEnterEvent,
  saveDisplayEvent
};

typedef struct
{
  enum events    eType;
  unsigned long  timeOfEvent;
  BOOLEAN        penDown;
  unsigned short ch;
  short          screenX;
  short          screenY;
  short          overlayX;
  short          overlayY;
  unsigned short data1;
  unsigned short data2;
  unsigned short data3;
  unsigned short data4;
  unsigned short data5;
  unsigned short data6;
} EVENTTYPE;


union TIMESPLITTYPE
{
  TIMETYPE   tt;
  struct
  {
    unsigned short   first;
    unsigned short   second;
  } tt_words;
};



extern void _APICALL EvtAddEventToQueue
 (EVENTTYPE far *event, unsigned short far *error);

extern unsigned long _APICALL EvtCurrentTick (void);

extern void _APICALL EvtGetEvent
 (EVENTTYPE far *event, unsigned short far *error);

extern unsigned short _APICALL EvtSetDoublePenDownTimeout 
 (unsigned short timeout);

extern unsigned short _APICALL EvtSetDoublePenDownRange (unsigned short range);

