/****************************************************************************
*
*	Module Name: gpdfld.h
*
*	Description:
*
*	Revision History:
*	0.6	02/10/95		FJL
*			Replaced "pascal" in all API function prototypes with _APICALL
*			which is defined as "pascal far" for Windows and "pascal" DOS.
*
****************************************************************************/

/*
 * $Workfile:   GPDFLD.H  $
 *
 * Current $Revision:   1.1  $ By $Author:   westerinen  $
 *
 * $Modtime:   03 Dec 1992 17:44:54  $
 *
 * Reverse Chronological Revision 
 *
 * $Log:   H:/penright/vcs/incs/gpdfld.h_v  $
 * 
 *    Rev 1.1   03 Dec 1992 17:44:58   westerinen
 * added vcs header
 *
*/


/*--------------------------------------------------------*/
/*                  Fields include file                   */
/*--------------------------------------------------------*/

/****************************************************************
 This include file uses the defined symbol KINDBTFLD to provide an
 alternative way to access the values encoded in FIELDTYPE.kind.
 When KINDBTFLD is defined the kind field is interpreted as a bit
 field and the members of FLDKINDBITS should be used to access the
 kind field rathar than the defines ALIGNMASK..DISPLAYCASEMASK.
 The KINDBTFLD is normally defined by setting a command line option
 for the compiler: -DKINDBTFLD
*******************************************************************/

/* access FIELDTYPE.kind as an unsigned short value                  */
/* masks for kind info                       bits used               */
#define ALIGNMASK             3   /* - - - - - - - - - - - - - - b b */
#define ACCENTMASK           60   /* - - - - - - - - - - b b b b - - */
#define OVERSTRIKEMASK       64   /* - - - - - - - - - b - - - - - - */
#define PROTECTEDMASK       128   /* - - - - - - - - b - - - - - - - */
#define SCROLLABLEMASK      256   /* - - - - - - - b - - - - - - - - */
#define EXPANDABLEMASK      512   /* - - - - - - b - - - - - - - - - */
#define INSPTDISABLEDMASK  1024   /* - - - - - b - - - - - - - - - - */
#define DISPLAYMINKBDMASK  2048   /* - - - - b - - - - - - - - - - - */
#define DISPLAYCASEMASK   12288   /* - - b b - - - - - - - - - - - - */

/* access FIELDTYPE.kind as a bit field value                        */
typedef struct kindBits
{
  unsigned  alignment:    2;
  unsigned  accent:       4;
  unsigned  overstrike:   1;
  unsigned  protect:      1;
  unsigned  scrollable:   1;
  unsigned  expandable:   1;
  unsigned  insertPtOff:  1;
  unsigned  keybdDisplay: 1;
  unsigned  caseDisplay:  2;
} FLDKINDBITS;

/* kind values */
#define LEFTALIGN        0
#define CENTERALIGN      1
#define RIGHTALIGN       2

#if defined(KINDBTFLD)
#define NOACCENT         0
#define UNDERLINEACCENT  1
#define COMBACCENT       2
#define BOXEDACCENT      3
#define FldAlignment(fld)  ((fld)->kind.alignment)
#define FldAccent(fld)     ((fld)->kind.accent)
#define FldOverStrike(fld) ((fld)->kind.overstrike)
#define FldProtected(fld)  ((fld)->kind.protect)
#define FldScrollable(fld) ((fld)->kind.scrollable)
#define FldExpandable(fld) ((fld)->kind.expandable)
#define FldInsPtDisabled(fld) ((fld)->kind.insertPtOff)
#define FldMiniKeyboard(fld) ((fld)->kind.keybdDisplay)
#define FldDisplayCase(fld) ((fld)->kind.caseDisplay)
#else
#define NOACCENT         0
#define UNDERLINEACCENT  4
#define COMBACCENT       8
#define BOXEDACCENT      12
#define FldAlignment(fld)  ((fld)->kind & ALIGNMASK)
#define FldAccent(fld)     ((fld)->kind & ACCENTMASK)
#define FldOverStrike(fld) (((fld)->kind & OVERSTRIKEMASK) == OVERSTRIKEMASK)
#define FldProtected(fld)  (((fld)->kind & PROTECTEDMASK) == PROTECTEDMASK)
#define FldScrollable(fld) (((fld)->kind & SCROLLABLEMASK) == SCROLLABLEMASK)
#define FldExpandable(fld) (((fld)->kind & EXPANDABLEMASK) == EXPANDABLEMASK)
#define FldInsPtDisabled(fld) (((fld)->kind & INSPTDISABLEDMASK)==INSPTDISABLEDMASK)
#define FldMiniKeyboard(fld) (((fld)->kind & DISPLAYMINKBDMASK)==DISPLAYMINKBDMASK)
#define FldDisplayCase(fld) ((fld)->kind & DISPLAYCASEMASK)
#endif

/* field highlight types */
#define NOHIGHLIGHT   0
#define OUTLINES      1
#define INVERTED      2

/* field case display types */
#if defined(KINDBTFLD)
#define ALLUPPER    0
#define FIRSTUPPER  1
#define MIXED       2
#else
#define ALLUPPER    0
#define FIRSTUPPER  4096
#define MIXED       8192
#endif

/* field data types */
#define STRINGDATA       0
#define NUMERICDATA      1
#define IMAGEDATA        2
#define POINTDATA        3
#define ALPHADATA        4
#define DIGITDATA        5

union FIELDDATATYPE
{
  char          ch [1];
  RSCCANVASHDR  image;
  POINTTYPE     points [1];
};

typedef struct
{
  unsigned short  charRangeStart;
  unsigned short  charRangeEnd;
} CHARRANGETYPE;

typedef struct
{
  unsigned short  length;
  CHARRANGETYPE   range [1];
} DISPLAYMASKTYPE;

typedef struct dl
{
  struct dl far  *next;
  unsigned short  start;
  unsigned short  length;
} DISPLAYLINETYPE;

typedef struct
{
  unsigned char  x;
  unsigned char  y;
} CELLEXTENTTYPE;

typedef struct
{
  unsigned short      userID;
  RECTANGLETYPE       rect;
  unsigned char       dataType;
#if defined(KINDBTFLD)
  FLDKINDBITS         kind;
#else  
  unsigned short      kind;
#endif
  unsigned short      dataLen;
  unsigned short      dataMax;
  void far            *pData;
  unsigned char       highlight;
  unsigned char       format;
  double              theNumber;
  unsigned short      font;
  char far            *displayMask;
  DISPLAYLINETYPE     display;
  CELLEXTENTTYPE      charCellExtent;
  unsigned short      selFirstPos;
  unsigned short      selLastPos;
  unsigned short      insPtXPos;
  unsigned short      insPtYPos;
  void far           *userData;
} FIELDTYPE;


enum fieldValueType { realValue = 0, integerValue, charValue, stringValue,
                      cStringValue, imageValue, pointsValue };

union FIELDVALUERECORD
{
  double             realNumber;
  unsigned long      integerNumber;
  unsigned char      ch;
  char far          *cString;
  RSCCANVASPTR       image;
  struct
  {
    char far        *ptr;
    unsigned short   len;
  } string;
  struct
  {
    POINTTYPE far   *pPts;
    unsigned short   len;
  } points;
};


extern short _APICALL FldCharWidth (FIELDTYPE far *fld);

extern void _APICALL FldCut
 (FIELDTYPE far *fld, unsigned short startPosition, unsigned short endPosition);

extern void _APICALL FldDrawField (FIELDTYPE far *fld);

extern void _APICALL FldEraseField (FIELDTYPE far *fld);

extern void _APICALL FldFreeFieldMemory
 (FIELDTYPE far *fld, unsigned short far *error);

extern void _APICALL FldGetFieldFrameRect
 (FIELDTYPE far *fld, RECTANGLETYPE far *rect);

extern DISPLAYMASKTYPE far * _APICALL FldGetMaskForChar
 (FIELDTYPE far *pField, WORD cellRow, WORD cellCol);

extern void _APICALL FldGetSelectedText
 (FIELDTYPE far *fld, char far * (far *pText), unsigned short far *len);

extern void _APICALL FldGetSelection
 (FIELDTYPE far *fld, unsigned short far *startPosition,
  unsigned short far *endPosition);

extern BOOLEAN _APICALL FldHandleKbKey
 (FIELDTYPE far *fld, unsigned short ch, unsigned short far *error);

extern void _APICALL FldInitField
 (FIELDTYPE far *fld, RECTANGLETYPE far *rect, unsigned char dataType,
  unsigned short alignment, unsigned short accent, BOOLEAN overstrike,
  BOOLEAN protect, BOOLEAN scrollable, unsigned short far *error);

extern short _APICALL FldLineHeight (FIELDTYPE far *fld);

extern void _APICALL FldPaste
 (FIELDTYPE far *fld, char far *pText, unsigned short len,
  unsigned short far *error);

extern void _APICALL FldResetField 
 (FIELDTYPE far *fld, unsigned short far *error);

extern void _APICALL FldScrollField
 (FIELDTYPE far *fld, unsigned short distance, enum directions direction);

extern void _APICALL FldSetSelection
 (FIELDTYPE far *fld, unsigned short startPosition,
  unsigned short endPosition);

extern void _APICALL FldStartSelection
 (FIELDTYPE far *fld, short x, short y);


/*----------------------*/
/* Field value routines */
/*----------------------*/

extern void far * _APICALL FldGetFieldData
 (FIELDTYPE far *fld, unsigned short far *len, unsigned short far *error);

extern void _APICALL FldGetFieldValue
 (FIELDTYPE far *fld, enum fieldValueType valueType,
  union FIELDVALUERECORD far *value, unsigned short far *error);

extern RSCCANVASHDR far * _APICALL FldGetImageValue
 (FIELDTYPE far *fld, unsigned short far *len, unsigned short far *error);

extern long _APICALL FldGetIntegerValue
 (FIELDTYPE far *fld, unsigned short far *error);

extern POINTTYPE far * _APICALL FldGetPointsValue
 (FIELDTYPE far *fld, unsigned short far *len, unsigned short far *error);

extern void _APICALL FldGetRealValue
 (FIELDTYPE far *fld, double far * num, unsigned short far *error);

extern char far * _APICALL FldGetStringValue
 (FIELDTYPE far *fld, unsigned short far *len, unsigned short far *error);


extern void _APICALL FldSetFieldData
 (FIELDTYPE far *fld, void far *data ,unsigned short len,
  unsigned short far *error);

extern void _APICALL FldSetFieldValue
 (FIELDTYPE far *fld, enum fieldValueType valueType,
  union FIELDVALUERECORD far *newValue, unsigned short far *error);

extern void _APICALL FldSetImageValue
 (FIELDTYPE far *fld, RSCCANVASHDR far *image, unsigned short far *error);

extern void _APICALL FldSetIntegerValue
 (FIELDTYPE far *fld, long num, unsigned short far *error);

extern void _APICALL FldSetPointsValue
 (FIELDTYPE far *fld, POINTTYPE far *points, unsigned short len,
  unsigned short far *error);

extern void _APICALL FldSetRealValue
 (FIELDTYPE far *fld, double num, unsigned short far *error);

extern void _APICALL FldSetStringValue
 (FIELDTYPE far *fld, char far *pText, unsigned short len,
  unsigned short far *error);


/*--------------------------*/
/* Insertion point routines */
/*--------------------------*/

extern void _APICALL FldPtToRowCol
 (FIELDTYPE far *fld, short x, short y, unsigned short far *row,
  unsigned short far *col);

extern void _APICALL FldRowColToPt
 (FIELDTYPE far *fld, unsigned short row, unsigned short col,
  short far *x, short far *y);

extern void _APICALL FldGetRowColOfTextPosition
 (FIELDTYPE far *fld, unsigned short pos, unsigned short far *row,
  unsigned short far *col, BOOLEAN far *visible);

extern unsigned short _APICALL FldGetTextPositionOfRowCol
 (FIELDTYPE far *fld, unsigned short row, unsigned short col);

extern void _APICALL FldGetInsPtPosition
 (FIELDTYPE far *fld, unsigned short far *row, unsigned short far *col);

extern void _APICALL FldSetInsPtPosition
 (FIELDTYPE far *fld, unsigned short row, unsigned short col,
  BOOLEAN absolutePos);


/*-----------------*/
/*  scrap routines */
/*-----------------*/

extern void _APICALL FldAllocateScrapMemory
 (unsigned short len, unsigned short far *error);

extern void _APICALL FldFreeScrapMemory (unsigned short far *error);

extern void _APICALL FldGetScrap
 (char far * (far *pText), unsigned short far *len);

extern void _APICALL FldGetScrapSize
 (unsigned short far *len, unsigned short far *maxLen);

extern void _APICALL FldSetScrap
 (char far *pText, unsigned short len, unsigned short far *error);

