/****************************************************************************
*
*	Module Name: gpdinit.h
*
*	Description:
*
*	Revision History:
*	0.5	11/23/94		FJL
*			Made the following changes:
*			1.	Added conditional compilation for Window's around prototypes
*				for ApiInitialize(), ApiMiniInit(), and ApiGetWindowHandle().
*				Identifier PRWIN is used to conditionally compile for
*				Windows.
*
*	0.5	01/11/95		ECM
*			Made the following changes:
*			1.	Added prototype for ApiMiniReinit(). This funtion was added
*				for the tool.
*
*	0.6	02/10/95		FJL
*			Replaced "pascal" in all API function prototypes with _APICALL
*			which is defined as "pascal far" for Windows and "pascal" DOS.
*
*	0.6	03/20/95		ECM
*			Made the following changes:
*			1.	Added prototype for ApiTerminate().
*
****************************************************************************/

/*
 * $Workfile:   gpdinit.h  $
 *
 * Current $Revision:   1.2  $ By $Author:   westerinen  $
 *
 * $Modtime:   05 Apr 1994 17:34:32  $
 *
 * Reverse Chronological Revision 
 *
 * $Log:   H:\penright\vcs\incs\gpdinit.h_v  $
 * 
 *    Rev 1.2   13 Apr 1994 14:18:52   westerinen
 * 
 * Added abstract types for form object handles.
 * 
 *    Rev 1.1   03 Dec 1992 17:49:18   westerinen
 * added vcs header
 *
*/


/*----------------------------------------------------------*/
/*               Initialization include file                */
/*----------------------------------------------------------*/

#define FALSE 0
#define TRUE  1

#define GRIDPADTYPE   0x24
#define GRID1520TYPE  0x34
#define GRID1530TYPE  0x74
#define OTHERTYPE     0x19

#define OVERLAY  0
#define PENPAD   1
#define MOUSE    2

enum orientations { north = 0, west, south, east };
enum rotations { ninety = 4, oneHundredEighty, twoHundredSeventy };

typedef unsigned char  BOOLEAN;
typedef unsigned char  BYTE;
typedef unsigned short WORD;
typedef unsigned long  TIMETYPE;

typedef WORD far *ERRORPTR;
typedef WORD far *OBJECTIDPTR;
typedef WORD far *FORMIDPTR;
typedef WORD OBJECTID;
typedef WORD FORMID;

typedef struct 
{
  short x;
  short y;
} POINTTYPE;

typedef struct 
{
  POINTTYPE topLeft;
  POINTTYPE extent;
} RECTANGLETYPE;

typedef struct
{
  short left;
  short top;
  short right;
  short bottom;
} ABSRECTTYPE;

/** System configuration resource structure ***/
typedef struct
{
  WORD     version;          /* PenRight version # */
  BYTE     backColor;        /* background color */
  BYTE     foreColor;        /* foreground color */
  WORD     inkWidth;         /* Ink Width  */
  WORD     insPtBlinkRate;   /* insert Point Interval (in ms) */  
  WORD     dblClickTimeout;  /* double click timeout (in ms) */
  WORD     dblClickRange;    /* double click limit range (in pixels) */
  WORD     maxPoints;        /* max points for hwr */
  WORD     maxStrokes;       /* max strokes for hwr */
  WORD     maxChars;         /* max chars for hwr */
  BYTE     orientation;      /* initial screen orientation */
  BOOLEAN  hwpEnabled;       /* handwriting enabled (1=TRUE, 0 = FALSE) */
  WORD     penUpTimeout;     /* penup timeout for handwriting (in ms) */
  WORD     penTapTimeout;    /* max pendown time for tap gesture (in ms) */
  WORD     penHoldTimeout;   /* time for pendown & hold gesture (in ms) */
} SYSCONFIGTYPE;


extern unsigned short _APICALL PadInitialize (void);

#ifdef PRWIN

extern unsigned int _APICALL ApiInitialize(HANDLE hInstance,
                                             HANDLE hPrevInstance,
                                             LPSTR lpszCmdParam,
														   int nCmdShow);
														
extern unsigned int _APICALL ApiMiniInit(HWND hToolWnd, HANDLE hInstance);
extern unsigned int _APICALL ApiMiniReinit(HWND hToolWnd, HANDLE hInstance);
extern void _APICALL ApiMiniTerminate(void);

extern HWND _APICALL ApiGetWindowHandle(void);
extern HDC _APICALL ApiGetRefreshHdc(void);
extern HDC _APICALL ApiGetVideoHdc(void);
extern BOOLEAN _APICALL ApiGetCursor(void);
extern HANDLE _APICALL ApiGetHinstance(void);
extern void _APICALL ApiTerminate(void);

#ifdef PRWCOLOR
/* Color control functions */
extern COLORREF _APICALL pascal FldSetFrameColor(COLORREF prwColor);
extern COLORREF _APICALL pascal FldSetTextColor(COLORREF prwColor);
extern COLORREF _APICALL pascal FldSetHighlightColor(COLORREF prwColor);

extern COLORREF _APICALL pascal LstSetFrameColor(COLORREF prwColor);
extern COLORREF _APICALL pascal LstSetTextColor(COLORREF prwColor);
extern COLORREF _APICALL pascal LstSetHighlightColor(COLORREF prwColor);
extern COLORREF _APICALL pascal LstSetHighTextColor(COLORREF prwColor);
extern COLORREF _APICALL pascal LstSetBackColor(COLORREF prwColor);

extern COLORREF _APICALL pascal CtlSetBtnTextColor(COLORREF prwColor);
extern COLORREF _APICALL pascal CtlSetBtnDisTextColor(COLORREF prwColor);
extern COLORREF _APICALL pascal CtlSetBtnFaceColor(COLORREF prwColor);
extern COLORREF _APICALL pascal CtlSetBtnShadowColor(COLORREF prwColor);
extern COLORREF _APICALL pascal CtlSetBtnHighColor(COLORREF prwColor);

extern COLORREF _APICALL pascal CtlSetRadioTextColor(COLORREF prwColor);
extern COLORREF _APICALL pascal CtlSetRadioColor(COLORREF prwColor);

extern COLORREF _APICALL pascal CtlSetCheckTextColor(COLORREF prwColor);
extern COLORREF _APICALL pascal CtlSetCheckColor(COLORREF prwColor);

extern COLORREF _APICALL pascal WinSetBackgroundColor(COLORREF prwColor);
extern COLORREF _APICALL pascal WinGetBackgroundColor(void);
#endif


#endif

extern void _APICALL PadTerminate (void);

extern unsigned char _APICALL PadGetComputerType (void);

extern unsigned char _APICALL PadGetPointingDevice (void);

extern unsigned char _APICALL PadApiInEms (void);

extern void _APICALL PadEnableEmsOptimization (BOOLEAN enabled);

extern unsigned short _APICALL PadSetInkWidth(unsigned short width);

extern unsigned short _APICALL PadGetInkWidth(void);

extern void _APICALL PadInitVideo(void);
