/****************************************************************************
*
*	Module Name: gpdkbnav.h
*
*	Description:
*
*	Revision History:
*	0.6	02/10/95		FJL
*			Replaced "pascal" in all API function prototypes with _APICALL
*			which is defined as "pascal far" for Windows and "pascal" DOS.
*
****************************************************************************/

/*
 * $Workfile:   gpdkbnav.h  $
 *
 * Current $Revision:   1.17  $ By $Author:   westerinen  $
 *
 * $Modtime:   24 May 1994 11:43:52  $
 *
 * Reverse Chronological Revision 
 *
 * $Log:   H:\penright\vcs\incs\gpdkbnav.h_v  $
 * 
 *    Rev 1.17   24 May 1994 12:51:06   westerinen
 * Changed prototype of KbdSetCursorToTab() to return a BOOLEAN.
 * 
 *    Rev 1.16   23 May 1994 13:39:52   tharakan
 * Changed the names on FORMOBJECTTYPE enum to OT_xxxx from frm, fld etc.
 * 
 *    Rev 1.15   20 May 1994 17:05:10   westerinen
 * Changed prototype for KbdUIEnable() to return a boolean.
 * 
 *    Rev 1.14   17 May 1994 10:59:44   tharakan
 * Changed unsigned to unsigned short in declarations
 * 
 *    Rev 1.13   12 May 1994 22:38:28   tharakan
 * 
 * Added KbdValidateFormObjectFlags and removed PATTERNTYPE typedef
 * 
 *    Rev 1.12   12 May 1994 16:20:54   rahman
 * 
 * PR3.6: Added define for KBNAV_POSTPENDN_ONCTL.
 * 
 *    Rev 1.11   11 May 1994 16:48:30   tharakan
 * Added KbdInitSequenceObject
 * 
 *    Rev 1.10   11 May 1994 09:47:58   woodmansee
 * KbdDrawMarquee and KbdEraseMarquee no long require objecttype to be 
 * passed
 * 
 *    Rev 1.9   10 May 1994 18:08:26   rahman
 * 
 * PR3.6: new definition NO_CUROBJIDX. currentObjIdx will get set to this
 * value if form has no focusEnabled object.
 * 
 *    Rev 1.8   09 May 1994 14:55:02   tharakan
 * 
 * Added KbdDrawMarquee and KbdEraseMarquee
 * 
 *    Rev 1.7   06 May 1994 19:26:30   westerinen
 * Fixed OBJFLAG_NOFOCUS to include NOT1STINGRP.
 * 
 *    Rev 1.6   05 May 1994 15:51:20   rahman
 * 
 * PR3.6: KbdUIDisable function returns a BOOLEAN.
 * 
 *    Rev 1.5   04 May 1994 18:39:32   westerinen
 * Added prototypes for KbdUIDisable() and KbdFocusEnableObjectID().
 * 
 *    Rev 1.4   04 May 1994 17:36:28   tharakan
 * Removed unused defines for OBJFLAG_xxxxxx
 * 
 *    Rev 1.3   02 May 1994 18:22:26   tharakan
 * Added define for element not first in grouped controls
 * 
 *    Rev 1.2   29 Apr 1994 17:03:14   tharakan
 * 
 * Added defines for objectFlags WORD
 * 
 *    Rev 1.1   26 Apr 1994 16:59:12   westerinen
 * 
 * Removed conditional inclusion for internal callers.
 * 
 *    Rev 1.0   13 Apr 1994 14:16:26   westerinen
 * Initial Revision
 *
*/


/****************************************************************************
*
*   Name: gpdkbnav.h
*
*   Description:
*     This file contains the PenRight! keyboard navigation data structure
*   definitions and the prototypes for the exported keyboard navigation 
*   functions.
*
****************************************************************************/

/* keyboard navigation data structures */

typedef enum {
    OT_FRM, 
    OT_FLD, 
    OT_PTFLD, 
    OT_BUTTN, 
    OT_RADIO, 
    OT_CHKBX, 
    OT_FRAME, 
    OT_RCTANG, 
    OT_TEXT, 
    OT_LINE, 
    OT_CANVAS, 
    OT_LIST, 
    OT_GROUP
}   FORMOBJECTTYPE;   /* type of element */

typedef struct {
    OBJECTID objectId;
    FORMOBJECTTYPE objectType;
    WORD objectFlags;
    BOOLEAN focusEnabled;
    WORD objectIndex;
    void far *reserved1;
    void far *reserved2;
} SEQUENCEOBJECT;

/* defines for the meaning of the objectFlags bits */
#define OBJFLAG_PROTECTED   0x0001  /* bit 0  ON => protected obj */
#define OBJFLAG_NOT1STINGRP 0x0040  /* bit 6  ON => Not the 1st obj in group */
#define OBJFLAG_CUSTOMCTRL  0x0080  /* bit 7  ON => custom control */
#define OBJFLAG_NONKBNAV    0x0100  /* bit 8  ON => unsupported obj */

/* convenience defines for combinations of the above */
/* if objectFlags & OBJFLAG_NOFOCUS then focusEnabled should be FALSE */
#define OBJFLAG_NOFOCUS     \
    (                       \
    OBJFLAG_PROTECTED   |   \
    OBJFLAG_NOT1STINGRP |   \
    OBJFLAG_CUSTOMCTRL  |   \
    OBJFLAG_NONKBNAV        \
    )

/* if objectFlags & OBJFLAG_NOCURRENT then KbdSetCurrentObjectID should NOP */
#define OBJFLAG_NOCURRENT   \
    (                       \
    OBJFLAG_PROTECTED       \
    )


typedef struct t_FORMOBJECTSEQ {
    struct t_FORMOBJECTSEQ far *next;
    FORMID formId;
    OBJECTID crObjId;
    OBJECTID escObjId;
    unsigned short numObjects;
    unsigned short currentObjIdx;
    SEQUENCEOBJECT far *objectSequence;
    void far *userData;
    BOOLEAN reserved1;
    BOOLEAN reserved2;
    void far *reserved3;
} FORMOBJECTSEQ;

#define NO_CUROBJIDX 0xFFFF

/* bit 15  ON => pendown event posted for kbdnav */
#define KBNAV_POSTPENDN_ONCTL 0x8000


typedef struct {
    unsigned short numForms;
    FORMOBJECTSEQ *pCurFormObjSeq;
    FORMOBJECTSEQ *formObjSeqList;
    BOOLEAN kbdSetCursorToTab;
    PATTERNTYPE marqueePattern;
    void far *userData;
    void far *reserved1;
    void far *reserved2;
} APPKBDFORMS;


/* keyboard navigation functions */

BOOLEAN _APICALL KbdUIEnable(void);

BOOLEAN _APICALL KbdUIDisable(void);

BOOLEAN _APICALL KbdGetNavEnable(void);

BOOLEAN _APICALL KbdSetNavEnable(BOOLEAN enabled);

BOOLEAN _APICALL KbdSetCursorToTab(BOOLEAN enabled, ERRORPTR pError);

APPKBDFORMS far * _APICALL KbdGetAppKbdFormPtr(void);

FORMOBJECTSEQ far * _APICALL KbdGetFormObjectSeqPtr(
    FORMID formId,
    ERRORPTR pError);

OBJECTID _APICALL KbdGetObjectID(
    FORMID formId,
    ERRORPTR pError);

OBJECTID _APICALL KbdGetCurrentObjectID(ERRORPTR pError);

OBJECTID _APICALL KbdGetCurrentObjectIDFormID(
    FORMIDPTR pFormId,
    ERRORPTR pError);

void _APICALL KbdSetCurrentObjectID(
    OBJECTID objectId,
    ERRORPTR pError);

void _APICALL KbdSetCurrentFormSeq(
    FORMID formId,
    ERRORPTR pError);

BOOLEAN _APICALL KbdFocusEnableCurrentObjectID(
    OBJECTID objectId, 
    BOOLEAN focusEnabled, 
    ERRORPTR pError
);
BOOLEAN _APICALL KbdFocusEnableObjectID(
    FORMID formId,
    OBJECTID objectId, 
    BOOLEAN focusEnabled, 
    ERRORPTR pError
);
OBJECTID _APICALL KbdSetFocusObjectIDNext(ERRORPTR pError);

OBJECTID _APICALL KbdSetFocusObjectIDPrev(ERRORPTR pError);

unsigned short _APICALL KbdGetSequenceObjectIndex(
    FORMID formId,
    OBJECTID objectId, 
    FORMOBJECTSEQ far * (far *ppFormObjectSeq),
    SEQUENCEOBJECT far * (far *ppSeqObject),
    ERRORPTR pError
);

void _APICALL KbdDrawMarquee(
    FORMID formId, 
    OBJECTID objectId, 
    PATTERNTYPE marqueePattern, 
    ERRORPTR pError
);


void _APICALL KbdEraseMarquee(
    FORMID formId, 
    OBJECTID objectId, 
    PATTERNTYPE marqueePattern, 
    ERRORPTR pError
);


void _APICALL KbdInitSequenceObject(
    FORMID formId,
    OBJECTID objectId,
    FORMOBJECTTYPE objectType,
    unsigned short objSeqIdx,
    ERRORPTR pError
);

void _APICALL KbdValidateFormObjectFlags(
    FORMID formId,
    ERRORPTR pError
);

