/****************************************************************************
*
*	Module Name: gpdrsc.h
*
*	Description:
*
*	Revision History:
*	0.6	02/10/95		FJL
*			Replaced "pascal" in all API function prototypes with _APICALL
*			which is defined as "pascal far" for Windows and "pascal" for DOS.
*
****************************************************************************/

/*
 * $Workfile:   gpdrsc.h  $
 *
 * Current $Revision:   1.3  $ By $Author:   westerinen  $
 *
 * $Modtime:   12 May 1993 11:40:18  $
 *
 * Reverse Chronological Revision 
 *
 * $Log:   H:\penright\vcs\incs\gpdrsc.h_v  $
 * 
 *    Rev 1.3   12 May 1993 11:44:06   westerinen
 * Broke out system resource definitions into a separate file, "sysrsc.h".
 * 
 *    Rev 1.2   23 Feb 1993 17:44:18   westerinen
 * Added system resource ID for form display code resource.
 * 
 *    Rev 1.1   03 Dec 1992 17:58:48   westerinen
 * added vcs header
 *
*/


/*--------------------------------------------------------*/
/*                  Resource include file                 */
/*--------------------------------------------------------*/

/** Resource IDs for built-in PenRight system resources  **/
#include <sysrsc.h>

enum resourceTypes
{
  codeRscType = 1,
  fontRscType,
  stringRscType,
  genericRscType,
  canvasRscType,
  formRscType
};

enum codeRscFunctions 
{ 
  rscTerminate = -2, 
  rscInit = -1, 
  rscCall = 0 
};

/** file header format for resource file ***/
typedef struct 
{
  unsigned short  signature;
  unsigned short  lenRscTable;
  unsigned short  numResources;
  unsigned char   version;
  unsigned char   futureUse [9];
} RSCHEADERTYPE;

typedef struct 
{
  unsigned short  rscID;        /* resource ID */
  unsigned short  rscType;      /* resource type */
  unsigned short  rscLength;    /* resource length */
  unsigned short  rscFlags;     /* flags - bit 1 = get flag */
  unsigned long   rscFilePos;   /* long offset into file */
  unsigned short  rscMemAddr;   /* segment of resource */
  unsigned short  useCount;     /* use counter */
} RSCENTRYTYPE;


/** internal resource file table format ***/
/* the size of the following structure varies on the number of entries */
typedef struct 
{
  WORD numEntries;          /* number of resource entries in this file */
  WORD conn;                /*  file handle (zero if closed) */
  WORD locks;               /* number of shared resources in file */
  WORD next;                /* segment of next resource file table */
  WORD reserved[4];
  RSCENTRYTYPE  entries[100]; /* table of resources in this file */
} RSCTABLETYPE;

typedef struct 
{
  unsigned short  signature;
  unsigned short  numStrings;
  unsigned short  unused [6];
  unsigned short  offsets [1];
  char            strEntries [1];
} RSCSTRINGTYPE;


extern unsigned short _APICALL RscOpenResourceFile
 (char far *pRscPathname, unsigned short far *pError);

extern void _APICALL RscCloseResourceFile
 (unsigned short resourceFileID, unsigned short far *pError);

extern void far * _APICALL RscGetResource
 (unsigned short rscID, unsigned short far *rscLength,
  unsigned short far *error);

extern void far * _APICALL RscGetResourceCopy
 (unsigned short rscID, unsigned short far *rscLength,
  unsigned short far *error);

extern void _APICALL RscFreeResource
 (void far *rscData, unsigned short far *error);

extern char far * _APICALL RscGetResourceString
 (void far *strRscData, unsigned short index, unsigned short far *strLength);

extern char far * _APICALL RscGetResourceStringCopy
 (void far *strRscData, unsigned short index, unsigned short far *strLength);

extern void far * _APICALL RscGetSharedResource
 (unsigned short rscID, unsigned short far *rscLength,
  unsigned short far *error);

extern void _APICALL RscReleaseSharedResource
 (void far *rscData, unsigned short far *error);

extern RSCTABLETYPE far * _APICALL RscGetResourceList (void);

