/****************************************************************************
*
*	Module Name: gpdwin.h
*
*	Description:
*
*	Revision History:
*	0.6	02/10/95		FJL
*			Replaced "pascal" in all API function prototypes with _APICALL
*			which is defined as "pascal far" for Windows and "pascal" for DOS.
*
****************************************************************************/

/*
 * $Workfile:   gpdwin.h  $
 *
 * Current $Revision:   1.2  $ By $Author:   westerinen  $
 *
 * $Modtime:   09 Jun 1994 14:07:42  $
 *
 * Reverse Chronological Revision 
 *
 * $Log:   H:\penright\vcs\incs\gpdwin.h_v  $
 * 
 *    Rev 1.2   10 Jun 1994 09:02:00   westerinen
 * Added prototype for WinGetVisibleDisplayExtent().
 * 
 *    Rev 1.1   03 Dec 1992 17:59:34   westerinen
 * added vcs header
 *
*/


/*--------------------------------------------------------*/
/*                 Windows include file                   */
/*--------------------------------------------------------*/

/*  Frame Types  */

#define NOFRAME       0
#define SIMPLEFRAME   1
#define PICTUREFRAME  41
#define SE3DSHADOW2   641

#define SAVE_DISPLAY     TRUE
#define RESTORE_DISPLAY  FALSE

/* The windowFlags fields has the following bits defined:              */
/*   Bit 0 is window format:  0=screen mode; 1=generic mode            */
/*   Bit 1 is offscreen flag: 0=onscreen window; 1=offscreen window    */
/*   Bit 2 is modal flag:     0=modeless window; 1=modal window        */
/*   Bit 3 is title flag:     0=no title area; 1=has title area        */
/*   Bit 4 is enabled flag:   0=window disabled; 1=window enabled      */
/*   Bit 5 is form win flag:  0=non-form window; 1=form's window       */
/*   Bit 6 is ems win flag:   0=conventional memory; 1=ems memory      */

#define WINFORMATMASK        0x01
#define WINOFFSCREENMASK     0X02
#define WINMODALMASK         0x04
#define WINTITLEMASK         0x08
#define WINENABLEDMASK       0x10
#define WINFORMWINDOWMASK    0x20
#define WINEMSMASK           0x40

enum directions { up = 0, down, left, right };
enum offscreenWindowFormats { screenFormat = 0, genericFormat };
enum patternXferModes { draw = 0, erase, invert, merge };

typedef unsigned char PATTERNTYPE [8];

typedef struct 
{
  unsigned short  displayWidth;
  unsigned short  displayHeight;
  unsigned short  displayAddr;
  unsigned short  windowFlags;
  RECTANGLETYPE   windowBounds;
  ABSRECTTYPE     clippingBounds;
  unsigned short  nextWindow;
  unsigned short  frameType;
  unsigned long   userData;
  char far        *pTitle;
  unsigned short  lenTitle;
  POINTTYPE       origin;
  unsigned short  font;
  unsigned short  reserved[10];
} WINDOWINFOTYPE;
#define WINDOWINFOPTR WINDOWINFOTYPE far *


/* Routines relating to windows */

extern unsigned short _APICALL WinCreateWindow
 (RECTANGLETYPE far *r, unsigned short frame, BOOLEAN modal, BOOLEAN titleArea,
  char far *titleChars, unsigned short titleLen, unsigned short far *error);

extern unsigned short _APICALL WinCreateOffscreenWindow
 (short width, short height,
  enum offscreenWindowFormats format, unsigned short far *error);

extern unsigned short _APICALL WinCreateMonOffscreenWindow
 (short width, short height,
  enum offscreenWindowFormats format, unsigned short far *error);

extern unsigned short _APICALL WinCreateEmsOffscreenWindow
 (short width, short height,
  enum offscreenWindowFormats format, unsigned short far *error);

extern void _APICALL WinDeleteWindow
 (unsigned short winHandle, BOOLEAN eraseIt, unsigned short far *error);

extern void _APICALL WinSetActiveWindow (unsigned short winHandle);

extern void _APICALL WinSetDrawWindow (unsigned short winHandle);

extern void _APICALL WinSetDisplayOrientation
 (enum orientations orientation);

extern unsigned short _APICALL WinGetDrawWindow (void);

extern unsigned short _APICALL WinGetActiveWindow (void);

extern unsigned short _APICALL WinGetDisplayWindow (void);

extern unsigned short _APICALL WinGetFirstWindow (void);

extern enum orientations _APICALL WinGetDisplayOrientation (void);

extern void _APICALL WinEnableWindow (unsigned short winHandle);

extern void _APICALL WinDisableWindow (unsigned short winHandle);

extern WINDOWINFOTYPE far * _APICALL WinGetWindowsPointer
 (unsigned short winHandle);

extern void _APICALL WinGetWindowFrameRect
 (unsigned short winHandle, RECTANGLETYPE far *r);

extern void _APICALL WinDrawWindowFrame (void);

extern void _APICALL WinEraseWindow (void);

extern unsigned short _APICALL WinSaveBits
 (RECTANGLETYPE far *source, unsigned short far *error);

extern void _APICALL WinRestoreBits
 (unsigned short winHandle, short destTopLeftX,
  short destTopLeftY, unsigned short far *error);

extern void _APICALL WinCopyRectangle
 (unsigned short srcWinHandle, unsigned short dstWinHandle,
  RECTANGLETYPE far *srcRect, short destTopLeftX, short destTopLeftY);

extern void _APICALL WinScrollRectangle
 (RECTANGLETYPE far *r, enum directions direction, short distance);

extern void _APICALL WinGetDisplayExtent
 (short far *extentX, short far *extentY);

extern void _APICALL WinGetVisibleDisplayExtent
 (short far *extentX, short far *extentY);

extern void _APICALL WinGetWindowExtent
 (short far *extentX, short far *extentY);

extern void _APICALL WinDisplayToWindowPt (short far *x, short far *y);

extern void _APICALL WinWindowToDisplayPt (short far *x, short far *y);

extern void _APICALL WinSetClip (RECTANGLETYPE far *r);

extern void _APICALL WinGetClip (RECTANGLETYPE far *r);

extern void _APICALL WinClipRectangle (RECTANGLETYPE far *r);

extern unsigned short _APICALL WinDisplayNotice 
 (RECTANGLETYPE far *r, char far *message, unsigned short msgLen, 
  char far *ctlOneText, unsigned short ctlOneTextLen, 
  char far *ctlTwoText, unsigned short ctlTwoTextLen, 
  void far *pIcon, unsigned short far *error);

extern void _APICALL WinCopyDisplay 
 (unsigned short winHandle, BOOLEAN direction, unsigned short far *error);

extern WORD _APICALL WinSetColor(WORD color);
