/////////////////////////////////////////////////////////////////////
//
//  chkdemo.c - Protected Mode demo program for "check" code resource
//
/////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <penright.h>
#include "chkdemo.h"
								  

/*-----------------------------------------------------------*/
/*                      StartApplication                     */
/*                                                           */
/*  This routine initializes the PenRight! API, opens the    */
/*  application's resource file, and posts an application    */
/*  enter event.  The resources of an application are        */
/*  typically appended to the end of the application's       */
/*  exe file.  However, for debugging purposes you may       */
/*  want to have the resources in a separate file.  If       */
/*  this routine does not find the resources in the exe      */
/*  file, it will look for them in a file with the same      */
/*  file name as the exe file and an extension of "rsc",     */
/*  in the same directory as the exe file.                   */
/*                                                           */
/*-----------------------------------------------------------*/
WORD StartApplication (char * exePath, WORD * error)
{
	WORD initError;
	WORD rscFileId;
	char * path;

	initError = PadInitialize ();
	if (initError)
	{
		printf ("PenRight! initialization error: %u\n", initError);
		exit (initError - 9000);
	}

	rscFileId = RscOpenResourceFile (exePath, error);
	if (*error)
	{
		path = strdup (exePath);
		if (path)
		{
			strcpy (&path[strlen(path)-3], "Rsc");
			rscFileId = RscOpenResourceFile (path, error);
			free (path);
		}
	}

	PostEvent (appEnterEvent, 0);
	return (rscFileId);
}


/*-----------------------------------------------------------*/
/*                      StopApplication                      */
/*                                                           */
/*  This routine terminates the PenRight! API and closes     */
/*  the application's resource file.                         */
/*                                                           */
/*-----------------------------------------------------------*/
void StopApplication (WORD rscFileId)
{
	WORD error;

	RscCloseResourceFile (rscFileId, &error);
	PadTerminate();
	exit (0);
}

/*-----------------------------------------------------------*/
/*                       CheckError                          */
/*                                                           */
/*  This routine checks the error code it is passed.  If     */
/*  the error code is not zero, an error message will be     */
/*  displayed in a dialog box.  The dialog box will contain  */
/*  two buttons: one to indicate that processing should      */
/*  continue, and one to indicate that processing should be  */
/*  terminated.  This routine will return TRUE if processing */
/*  should be terminated, or FALSE if processing should      */
/*  continue.                                                */
/*                                                           */
/*-----------------------------------------------------------*/
BOOLEAN CheckError (WORD error)
{
	WORD ctlPressed;
	WORD localError;
	char msg [100];
	char * ctl1   = "Continue";
	char * ctl2   = "Exit";
	char * title  = "Warning";

	if (error == 0)
		return (FALSE);
	else
	{
		if (error <= sys_nerr)
			sprintf (msg, "Error %d: %.80s", error, strerror(error));
		else
			sprintf (msg, "PenRight! Error %d", error);
		ctlPressed = WinDisplayNotice (NULL, msg, strlen(msg), ctl1,
			     strlen(ctl1), ctl2, strlen(ctl2), title, &localError);
		if (! localError)
			return (ctlPressed == 2);
		printf (msg);
		return (TRUE);
	}
}


/*-----------------------------------------------------------*/
/*                          M A I N                          */
/*-----------------------------------------------------------*/
void main (int argc, char *argv[])
{
   WORD			error, frm;
   EVENTTYPE	event;
   BOOLEAN		done = FALSE;
   int			i, value;
	WORD			rscFileId;  /* handle of application's resource file */

	error = 0;
	
	if (argc)
		rscFileId = StartApplication (argv[0], &error);

   if(!CheckError(error))
	{
		frm = FrmInitForm(CHECK_FRM, &error);

		FrmDrawForm(frm, TRUE, TRUE);
		FrmSetActiveForm(frm);

		while( !done )
		{
		  
			EvtGetEvent(&event, &error);
			switch( event.eType )
			{
				case ctlExitEvent: if( event.data1 == EXIT_BTN )
				{
					done = TRUE;
				}
				break;

			}
			FrmHandleEvent(&event, &error);
		}
	}
	
	StopApplication (rscFileId);
	return(0);
}


