/*-----------------------------------------------------------*/
/*                                                           */
/*  Module Name: house.c                                     */
/*                                                           */
/*  Description:                                             */
/*    This is the application's main module.  This module    */
/*    starts the application, dispatches events, and stops   */
/*    the application.                                       */
/*                                                           */
/*  Edit History:                                            */
/*     1-24-95  Created                                      */
/*                                                           */
/*-----------------------------------------------------------*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "penright.h"
#include "house.h"

/* #define PRDatabase */
/* #define PRDatacomm */
/* #define DBSTUB */

#ifdef PRDatabase

/*   DBTEST variables */
CODE4 cb;          /* CodeBase data structure */
DATA4 *data;       /* Database data structure */

static FIELD4INFO field_info[] =
{
     /* field name, type, width, decimals */
     /* {"FNAME",    'C',  10,     0    },
        {"LNAME",    'C',  10,     0    },  */
        {0,0,0,0},     
};

static TAG4INFO tag_info[] = 
{
     /* name,  expression,  filter, unique, descending */
     /* {"FIRST", "FNAME",  "", r4unique_continue, 0},
        {"LAST", "LNAME", "", r4unique_continue, 0}, */
        {0,0,0,0,0},
};

PR_DBMMAP mapinfo [] =
{
     /* FormID,  ObjectID,  ObjectType,  DBField */
     /* {QueryFrm, FNameFld, Field, "FNAME"},
        {QueryFrm, LNameFld, Field, "LNAME"}, */
        {0,0,0,0},     
};

PR_DBMDATA dbinfo = {&cb, "DBNAME", mapinfo};

PR_MAPPTR  dbMapPtr = mapinfo;
PR_DBMPTR  dbDataPtr = &dbinfo;

/* For Microsoft, the stack length needs to be set as a linker option. */
/* __BORLANDC is a global variable defined by the compiler.            */
#ifdef __BORLANDC__
 extern unsigned _stklen = 12000;
#endif /* BORLANDC */

#endif /* PRDatabase */


#ifdef PRDatacomm

DCMPORT     DcmPort;
HDCMPORT    hDcmPort = &DcmPort;

#endif /* PRDatacomm */

static BOOLEAN oldKbnav = 0;

/*-----------------------------------------------------------*/
/*                      StartApplication                     */
/*                                                           */
/*  This routine initializes the PenRight! API, opens the    */
/*  application's resource file, and posts an application    */
/*  enter event.  The resources of an application are        */
/*  typically appended to the end of the application's       */
/*  exe file.  However, for debugging purposes you may       */
/*  want to have the resources in a separate file.  If       */
/*  this routine does not find the resources in the exe      */
/*  file, it will look for them in a file with the same      */
/*  file name as the exe file and an extension of "rsc",     */
/*  in the same directory as the exe file.                   */
/*                                                           */
/*-----------------------------------------------------------*/
WORD StartApplication (char * exePath, WORD * error)
{
	WORD initError;
	WORD rscFileId;
	char * path;

//	oldKbnav = KbdUIEnable ();
	initError = PadInitialize ();
	if (initError)
	{
		printf ("PenRight! initialization error: %u\n", initError);
		exit (initError - 9000);
	}

	rscFileId = RscOpenResourceFile (exePath, error);
	if (*error)
	{
		path = strdup (exePath);
		if (path)
		{
			strcpy (&path[strlen(path)-3], "Rsc");
			rscFileId = RscOpenResourceFile (path, error);
			free (path);
		}
	}

	PostEvent (appEnterEvent, 0);
	return (rscFileId);
}

/*-----------------------------------------------------------*/
/*                      StopApplication                      */
/*                                                           */
/*  This routine terminates the PenRight! API and closes     */
/*  the application's resource file.                         */
/*                                                           */
/*-----------------------------------------------------------*/
void StopApplication (WORD rscFileId)
{
	WORD error;

	RscCloseResourceFile (rscFileId, &error);
	PadTerminate();
//	if (!oldKbnav) KbdUIDisable ();
	exit (0);
}

/*-----------------------------------------------------------*/
/*                       CheckError                          */
/*                                                           */
/*  This routine checks the error code it is passed.  If     */
/*  the error code is not zero, an error message will be     */
/*  displayed in a dialog box.  The dialog box will contain  */
/*  two buttons: one to indicate that processing should      */
/*  continue, and one to indicate that processing should be  */
/*  terminated.  This routine will return TRUE if processing */
/*  should be terminated, or FALSE if processing should      */
/*  continue.                                                */
/*                                                           */
/*-----------------------------------------------------------*/
BOOLEAN CheckError (WORD error)
{
	WORD ctlPressed;
	WORD localError;
	char msg [100];
	char * ctl1   = "Continue";
	char * ctl2   = "Exit";
	char * title  = "Warning";

	if (error == 0)
		return (FALSE);
	else
	{
		if (error <= sys_nerr)
			sprintf (msg, "Error %d: %.80s", error, strerror(error));
		else
			sprintf (msg, "PenRight! Error %d", error);
		ctlPressed = WinDisplayNotice (NULL, msg, strlen(msg), ctl1,
			     strlen(ctl1), ctl2, strlen(ctl2), title, &localError);
		if (! localError)
			return (ctlPressed == 2);
		return (TRUE);
	}
}


/*------------------------------------------------------------*/
/*                        GetObjectId                         */
/*                                                            */
/*  This routine returns the id of the object (field, list,   */
/*  or control) in which the event occured.  If the event     */
/*  did not occur in a field, list, or control then the       */
/*  return value will be zero.                                */
/*                                                            */
/*------------------------------------------------------------*/
WORD GetObjectId (EVENTTYPE *event)
{
	WORD curFrm;
	WORD curFld;

	switch (event->eType)
	{
		case ctlEnterEvent:
		case ctlExitEvent:
		case lstEnterEvent:
		case lstChangedEvent:
		case lstExitEvent:
		case fldEnterEvent:
		case fldChangedEvent:
		case ctlPressedEvent:
			return (event->data1);

		case keyDownEvent:
			curFrm = WinGetActiveWindow ();
			curFld = FrmGetCurrentField (curFrm);
			if (curFld)
				return (FrmGetFieldId (curFrm, curFld));
			else
				return (0);

		default:
			return (0);
	}
}

/*-----------------------------------------------------------*/
/*                 Kitchen1NextButtonHandler                 */
/*-----------------------------------------------------------*/
BOOLEAN Kitchen1NextButtonHandler (EVENTTYPE *event, WORD *error)
{
	BOOLEAN handled = FALSE;

	*error = 0;
	switch (event->eType)
	{
		case ctlExitEvent:
			GotoForm (Kitchen2);
			break;
	}
	return (handled);
}

/*-----------------------------------------------------------*/
/*                    Kitchen1HandleEvent                    */
/*-----------------------------------------------------------*/
BOOLEAN Kitchen1HandleEvent (EVENTTYPE *event, WORD *error)
{
	BOOLEAN handled = FALSE;

	*error = 0;
	switch (GetObjectId (event))
	{
		case nextButton:
			handled = Kitchen1NextButtonHandler (event, error);
			break;
	}
	return (handled);
}

/*-----------------------------------------------------------*/
/*               Kitchen2PreviousButtonHandler               */
/*-----------------------------------------------------------*/
BOOLEAN Kitchen2PreviousButtonHandler (EVENTTYPE *event, WORD *error)
{
	BOOLEAN handled = FALSE;

	*error = 0;
	switch (event->eType)
	{
		case ctlExitEvent:
			GotoForm (Kitchen1);
			break;
	}
	return (handled);
}

/*-----------------------------------------------------------*/
/*                 Kitchen2DoneButtonHandler                 */
/*-----------------------------------------------------------*/
BOOLEAN Kitchen2DoneButtonHandler (EVENTTYPE *event, WORD *error)
{
	BOOLEAN handled = FALSE;

	*error = 0;
	switch (event->eType)
	{
		case ctlExitEvent:
			Stop ();
			break;
	}
	return (handled);
}

/*-----------------------------------------------------------*/
/*                    Kitchen2HandleEvent                    */
/*-----------------------------------------------------------*/
BOOLEAN Kitchen2HandleEvent (EVENTTYPE *event, WORD *error)
{
	BOOLEAN handled = FALSE;

	*error = 0;
	switch (GetObjectId (event))
	{
		case previousButton:
			handled = Kitchen2PreviousButtonHandler (event, error);
			break;
		case doneButton:
			handled = Kitchen2DoneButtonHandler (event, error);
			break;
	}
	return (handled);
}

/*-----------------------------------------------------------*/
/*                   CurrentFormHandleEvent                  */
/*-----------------------------------------------------------*/
BOOLEAN CurrentFormHandleEvent (EVENTTYPE *event, WORD *error)
{
	BOOLEAN handled = FALSE;
	WORD formRscId;

	*error = 0;
	formRscId = GetEventsObject(event);
	switch (formRscId)
	{
		case Kitchen1:
			handled = Kitchen1HandleEvent (event, error);
			break;
		case Kitchen2:
			handled = Kitchen2HandleEvent (event, error);
			break;
	}
	return (handled);
}

/*-----------------------------------------------------------*/
/*                   ApplicationHandleEvent                  */
/*-----------------------------------------------------------*/
BOOLEAN ApplicationHandleEvent (EVENTTYPE *event, WORD *error)
{
	BOOLEAN handled = FALSE;

	*error = 0;
	switch (event->eType)
	{
		case appEnterEvent:
			GotoForm (Kitchen1);
			break;
	}
	return (handled);
}


/*-----------------------------------------------------------*/
/*                     CodeStartHandleEvent                  */
/*-----------------------------------------------------------*/
BOOLEAN CodeStartHandleEvent (EVENTTYPE *event, WORD *error)
{
	BOOLEAN handled = FALSE;

	*error = 0;
	switch (event->eType)
	{
		case frmInitEvent:
			InitForm (event->data1);
			*error = GetErrorCode ();
			break;

		case frmOpenEvent:
			OpenForm (event->data1);
			*error = GetErrorCode ();
			break;

		case frmCloseEvent:
			CloseForm (event->data1);
			break;
	}
	return (handled);
}

/*-----------------------------------------------------------*/
/*                        EventLoop                          */
/*-----------------------------------------------------------*/
void EventLoop (void)
{
	WORD error;
	EVENTTYPE event;
	BOOLEAN handled;

	do
	{
		EvtGetEvent (&event, &error);
		if (CheckError (error)) break;
		
		handled = CurrentFormHandleEvent (&event, &error);
		if (CheckError (error)) break;

		if (! handled)
			handled = ApplicationHandleEvent (&event, &error);
		if (CheckError (error)) break;

		if (! handled)
			handled = CodeStartHandleEvent (&event, &error);
		if (CheckError (error)) break;

		if (! handled)
			handled = FrmHandleEvent (&event, &error);
		if (CheckError (error)) break;
	}
	while (event.eType != appExitEvent);
}

/*-----------------------------------------------------------*/
/*                          M A I N                          */
/*-----------------------------------------------------------*/
void main (int argc, char *argv[])
{
	WORD error = 0;
	WORD rscFileId;  /* handle of application's resource file */

	if (argc)
		rscFileId = StartApplication (argv[0], &error);

	if (! CheckError (error))
		EventLoop ();

	StopApplication (rscFileId);
}
