/*---------------------------------------------------------------*/
/* Name: Test.C                                                  */
/*                                                               */
/* Function: This module contains routines that test the scroll  */
/*           bar code resource.                                  */
/*                                                               */
/*---------------------------------------------------------------*/

#include <windows.h>
#include <penapi.h>
#include <verbs.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <dos.h>
#include <dir.h>

//#include <stdlib.h>
//#include <string.h>
//#include <dos.h>
//#include <dir.h>
//#include <gpadapi.h>

#include "test.h"

#define escapeKey 27
#define FunctionList 1000

char * rscPath;

/* ----- Forward Declarations ---------*/
WORD StartApplication (HANDLE hInstance, HANDLE hPrevInstance,
                       LPSTR lpszCmdParam, int nCmdShow, WORD *error);
void StopApplication (WORD rscFileId);
BOOLEAN CheckError (WORD error);

BOOLEAN ScrollListHandler (EVENTTYPE * event, WORD * error);
void ScrollListUpdate (WORD frmId, WORD id);

void EventLoop (void);

/*------------------------------------------------------------*/
/*                         CenterForm                         */
/*------------------------------------------------------------*/
void CenterForm (WORD frmRscId)
{
	WINDOWINFOTYPE far * windowPtr;
	short x, y, deltaX, deltaY;
	WORD winHandle;
	RECTANGLETYPE r;
	
	winHandle = FrmGetWindowHandle (frmRscId);
	windowPtr = WinGetWindowsPointer (winHandle);
	FrmGetFormFrameRect (winHandle, &r);
	x = windowPtr->displayWidth;
	y = windowPtr->displayHeight;
	x = (x - r.extent.x) / 2;
	y = (y - r.extent.y) / 2;
	deltaX = windowPtr->windowBounds.topLeft.x - r.topLeft.x;
	deltaY = windowPtr->windowBounds.topLeft.y - r.topLeft.y;
	windowPtr->windowBounds.topLeft.x = x + deltaX;
	windowPtr->windowBounds.topLeft.y = y + deltaY;
}	

/*-----------------------------------------------------------*/
/*                       CheckError                          */
/*                                                           */
/*  This routine checks the error code it is passed.  If     */
/*  the error code is not zero, an error message will be     */
/*  displayed in a dialog box.  The dialog box will contain  */
/*  two buttons: one to indicate that processing should      */
/*  continue, and one to indicate that processing should be  */
/*  terminated.  This routine will return TRUE if processing */
/*  should be terminated, or FALSE if processing should      */
/*  continue.                                                */
/*                                                           */
/*-----------------------------------------------------------*/
BOOLEAN CheckError (WORD error)
{
	WORD ctlPressed;
	WORD localError;
	char msg [100];
	char * ctl1   = "Continue";
	char * ctl2   = "Exit";
	char * title  = "Warning";

	if (error == 0)
		return (FALSE);
	else
	{
		if (error <= sys_nerr)
			sprintf (msg, "Error %d: %.80s", error, strerror(error));
		else
			sprintf (msg, "PenRight! Error %d", error);
		ctlPressed = WinDisplayNotice (NULL, msg, strlen(msg), ctl1,
			     strlen(ctl1), ctl2, strlen(ctl2), title, &localError);
		if (! localError)
			return (ctlPressed == 2);
		printf (msg);
		return (TRUE);
	}
}

/*------------------------------------------------------------*/
/*                        EventLoop                           */
/*------------------------------------------------------------*/
void EventLoop ()
{
	WORD error;
	EVENTTYPE event;
	BOOLEAN done;
	
	do
		{
		EvtGetEvent (&event, &error);
		CheckError (error);
		
		done = ScrollListHandler (&event, &error);
		CheckError (error);
		} 
	while (! done);
}

/*-----------------------------------------------------------*/
/*                       SOpenForm                            */
/*-----------------------------------------------------------*/
void SOpenForm (WORD frmRscId, WORD * error)
{
	WORD formId;
	
	formId = FrmGetWindowHandle (frmRscId);
	if (formId == 0)
		formId = FrmInitForm (frmRscId, error);
	FrmSetActiveForm (formId);	
	CenterForm (frmRscId);
	ScrollListUpdate (frmRscId, FunctionList);
	FrmDrawForm (formId, TRUE, TRUE);
}

/*-----------------------------------------------------------------------*/
/*                          ScrollListHandleEvent                        */
/*																		 */
/*	This routine links scroll bars to lists in a form. The link is made  */
/*	if a list and a scroll bar have the same Id value. Call this routine */
/*	in your event loop to update list or scroll bars automatically.      */
/*	The Scroll bar should first be set to match the list data by calling */
/*	ScrollListUpdate();                                                  */
/*-----------------------------------------------------------------------*/
BOOLEAN ScrollListHandleEvent (EVENTTYPE far *event, WORD far *error)
{	
	WORD frm, index;
	short distance, direction;
	LISTTYPE far *lst;
	CONTROLTYPE far *ctl;
	BOOLEAN handled = FALSE;		/* THIS ROUTINE ALWAYS RETURNS FALSE */
								/* the api must see both these events !!! */
	*error = 0;
	switch (event->eType)
		{	
		case ctlPressedEvent:
			frm = WinGetActiveWindow ();	/* get current form */
			
			/* if there is a list with the same ID as the control */
			if (index = FrmGetListIndex (frm, event->data1))
				{ 	/* get a pointer to the list */
				lst = FrmGetListPtr (frm, index);
				
				/* distance = new control value - old control value */
				distance = (short)event->data5 - (short)event->data3;
				if (distance < 0) 
					{	
					distance = -distance;
					direction = down;
					} 
				else
					direction = up;
				LstScrollList (lst, distance, direction);
				}
			break;

		case lstChangedEvent:
			/* if a control exists with same Id as the form */
			frm = WinGetActiveWindow ();	/* get current form */
			
			if (index = FrmGetControlIndex (frm, event->data1))
				{	
				ctl = FrmGetControlPtr (frm, index);
				
				/* get list pointer - data2 holds index */
				lst = FrmGetListPtr (frm, event->data2);
				CtlSetControlValue (ctl, lst->topItem);
				CtlDrawControl (ctl, TRUE); 
				}
			break;
		}
	return(handled);
}

/*------------------------------------------------------------*/
/*                    ScrollListHandler                       */
/*------------------------------------------------------------*/
BOOLEAN ScrollListHandler (EVENTTYPE * event, WORD * error)
{
	BOOLEAN done;
	BOOLEAN handled;
	WORD winHandle;
	
	done = FALSE;
	handled = FALSE;

	if (event->eType == ctlExitEvent)
		{
		if (event->data1 == EXIT)
			{
			done = TRUE;
			handled = TRUE;
			}
		}
	else
		handled = ScrollListHandleEvent (event, error);

	if (! handled)
		FrmHandleEvent (event, error);
		
	return (done);
}

/*----------------------------------------------------------------------*/
/*							 ScrollListUpdate 							*/
/*																		*/
/* This function will update a scroll bar to reflect the state of a		*/
/* list. The scroll bar max value is set to the current number of extra */
/* list elements and the page size is set to visible items.				*/
/* NOTE: this does NOT draw the control.								*/	
/*----------------------------------------------------------------------*/
void ScrollListUpdate (WORD frmId, WORD id)
{	
	LISTTYPE far *lst;
	CONTROLTYPE far *ctl;
	WORD frm, saveFont;
	short visLines;

	frm = FrmGetWindowHandle (frmId);
	lst = FrmGetListPtr (frm, FrmGetListIndex (frm, id));
	ctl = FrmGetControlPtr (frm, FrmGetControlIndex (frm, id));
	if (lst && ctl)	 /* if both elements are found */
		{	/* Calculate # of visible lines in list */
		saveFont=FntSetFontHandle(lst->font);
		visLines = lst->rect.extent.y / (FntLineHeight()+LSTLINEGAP);
		FntSetFontHandle(saveFont);
		ctl->minValue = 1;
		ctl->maxValue = lst->numItems-visLines+1;
		if (ctl->maxValue < 0) ctl->maxValue = 1;
		ctl->userData = visLines;	/* set page size */
		CtlSetControlValue(ctl,lst->topItem);
		}
}

/*-----------------------------------------------------------*/
/*                      SetRscPath                           */
/*-----------------------------------------------------------*/
void SetRscPath (char * programPath)
{
	WORD fileId;
	WORD error;
	char path [MAXPATH];
	char drive [MAXDRIVE];
	char dir [MAXDIR];
	char name [MAXFILE];
	char ext [MAXEXT];

	fileId = RscOpenResourceFile (programPath, &error);
	if (error == 0)
		rscPath	= strdup (programPath);
	else
  	{
		/* alternate resource file for debuging */
		fnsplit (programPath, drive, dir, name, ext);
		fnmerge (path, drive, dir, name, ".rsc");
		rscPath	= strdup (path);
		}
	RscCloseResourceFile (fileId, &error);
}
								  
int PASCAL WinMain (HANDLE hInstance, HANDLE hPrevInstance,
                    LPSTR lpszCmdParam, int nCmdShow)
{
   WORD			error, frm;
   EVENTTYPE	event;
   int			i, value;
	WORD			rscFileId;  /* handle of application's resource file */
	
	rscFileId = StartApplication (hInstance, hPrevInstance, lpszCmdParam,
											nCmdShow, &error);

	SOpenForm (ScrollForm, &error);
	CheckError (error);
	
	EventLoop ();
										
	StopApplication (rscFileId);
	return(0);

}



/*-----------------------------------------------------------*/
/*                      StartApplication                     */
/*                                                           */
/*  This routine initializes the PenRight! API, opens the    */
/*  application's resource file, and posts an application    */
/*  enter event.  The resources of an application are        */
/*  typically appended to the end of the application's       */
/*  exe file.  However, for debugging purposes you may       */
/*  want to have the resources in a separate file.  If       */
/*  this routine does not find the resources in the exe      */
/*  file, it will look for them in a file with the same      */
/*  file name as the exe file and an extension of "rsc",     */
/*  in the same directory as the exe file.                   */
/*                                                           */
/*-----------------------------------------------------------*/
WORD StartApplication (HANDLE hInstance, HANDLE hPrevInstance,
                       LPSTR lpszCmdParam, int nCmdShow, WORD *error)
{
	WORD initError;
	WORD rscFileId;
	char far *path;
	static char exePath[255];
	char msg [100];
	
	initError = ApiInitialize(hInstance, hPrevInstance, lpszCmdParam,
									  nCmdShow);
								  
	if (initError)
	{
		sprintf (msg, "PenRight! initialization error: %u\n", initError);
		MessageBox(0, msg, 0, MB_OK | MB_ICONEXCLAMATION | MB_TASKMODAL);
		DestroyWindow(ApiGetWindowHandle());
		exit (initError - 9000);
	}

	GetModuleFileName(hInstance, exePath, sizeof(exePath));
	
	rscFileId = RscOpenResourceFile (exePath, error);
	if (*error)
	{
		path = strdup(exePath);
		if (path)
		{
			strcpy (&path[strlen(path)-3], "Rsc");
			rscFileId = RscOpenResourceFile (path, error);
			free (path);
		}
	}

	PostEvent (appEnterEvent, 0);
	return (rscFileId);
}


/*-----------------------------------------------------------*/
/*                      StopApplication                      */
/*                                                           */
/*  This routine terminates the PenRight! API and closes     */
/*  the application's resource file.                         */
/*                                                           */
/*-----------------------------------------------------------*/
void StopApplication (WORD rscFileId)
{
	WORD error;

	RscCloseResourceFile (rscFileId, &error);
	DestroyWindow(ApiGetWindowHandle());
	exit (0);
}

